% Learning & forgetting. Besanko Doraszelski Satterthwaite (2004) 
% Distr(M+1,M+1,K) - set of disributions
% Stats(k).HerfP = herfindahl, cond. on firms winning 
%      .PrC = p/c, [leader follower average]  ...
%      .Mrg = (p-c)/sigma, [leader follower average] 

function Stats = DistrStatsNE(Distr)
global p ProbC stay
global M
global sigma vOut

% get K = number of distributions
tmp=size(Distr); 
if length(tmp)==3
    K = tmp(3);
else
    error('DistrStatsEE: input array (distributions) must be 3-dim: (M+1)*(M+1)*K');
end
if ( tmp(1) ~= M+1 ) | ( tmp(2)~=M+1 )
    error('DistrStatsEE: input array must have dimensions 1&2 at (M+1)')
end

% create output data stru
Stats = repmat(struct('HerfP',NaN, 'PrC',[NaN,NaN,NaN], ...
                      'Mrg',[NaN,NaN,NaN] ), K,1);

Distr0=Distr; % replace NaN with 0 in input
z=find(isnan(Distr0));
Distr0(z)=0;

%%% by state matrices
tmHerfP = zeros(M+1);  
tmPrC_L = zeros(M+1); tmPrC_F = zeros(M+1); tmPrC_A=zeros(M+1); % P/C ratio
tmMrg_L = zeros(M+1); tmMrg_F = zeros(M+1); tmMrg_A=zeros(M+1); % (P-C)/sigma difference
c = cost(1:M);

% get distribution matrix by intermediate state (price competition)
for i=1:M+1
for j=1:i % j<=i
    % precalculate win probabilities
    F = Fsale( p(i,j), p(j,i) );
    F0 = Fsale0( p(i,j), p(j,i) );
    
    % Herfindahl based on market shares
    tmHerfP(i,j) = ( F^2 + (1-F-F0)^2 ) / ((1-F0)^2);
    if i+j == 2*M+2 % i=j=M+1 - empty state
        tmHerfP(i,j) = 0.5;
    end
    
    % price/cost ratio
    if i < (M+1) % duopoly, i is the weak leader
        tmPrC_L(i,j) = p(i,j)/c(i); 
        tmPrC_F(i,j) = p(j,i)/c(j); 
        tmPrC_A(i,j) = ( tmPrC_L(i,j)*F + tmPrC_F(i,j)*(1-F-F0) ) / (1-F0);
    elseif j < (M+1) % i is out, j is the monopolist
        tmPrC_L(i,j) = p(j,i)/c(j); 
        tmPrC_F(i,j) = 0; 
        tmPrC_A(i,j) = tmPrC_L(i,j); 
    %else %i==j==M+1, empty industry - all three remain at zero
    end
    
    % (price-cost)
    if i < (M+1) % duopoly, i is the weak leader
        tmMrg_L(i,j) = (p(i,j)-c(i))/sigma; 
        tmMrg_F(i,j) = (p(j,i)-c(i))/sigma; 
        tmMrg_A(i,j) = ( tmMrg_L(i,j)*F + tmMrg_F(i,j)*(1-F-F0) ) / (1-F0);
    elseif j < (M+1) % i is out, j is the monopolist
        tmMrg_L(i,j) = (p(j,i)-c(j))/sigma; 
        tmMrg_F(i,j) = 0; 
        tmMrg_A(i,j) = tmMrg_L(i,j);
    %else %i==j==M+1, empty industry - all three remain at zero
    end
    
    if j<i % replicate the symmetric states
        tmHerfP(j,i) = tmHerfP(i,j);
        tmMrg_L(j,i) = tmMrg_L(i,j); tmMrg_F(j,i)=tmMrg_F(i,j); 
        tmMrg_A(j,i) = tmMrg_A(i,j); 
        tmPrC_L(j,i) = tmPrC_L(i,j); tmPrC_F(j,i)=tmPrC_F(i,j); 
        tmPrC_A(j,i) = tmPrC_A(i,j);
    end
end % j
end % i

% get stats by intial state (before exit/entry)
mHerfP = zeros(M+1);  
mPrC_L = zeros(M+1); mPrC_F = zeros(M+1); mPrC_A=zeros(M+1); % P/C ratio
mMrg_L = zeros(M+1); mMrg_F = zeros(M+1); mMrg_A=zeros(M+1); % (P-C)/sigma difference

for i=1:M+1
for j=1:i
   if i<M+1 % active state of 1st firm (non-exit or entry)
       ip=i;
   else
       ip=1;
   end
   if j<M+1 % active state of 2nd firm (non-exit or entry)
       jp=j;
   else
       jp=1;
   end
    
    mHerfP(i,j) =      stay(i,j)  *    stay(j,i)  * tmHerfP( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmHerfP(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmHerfP( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmHerfP(M+1,M+1);
              
    mMrg_L(i,j) =      stay(i,j)  *    stay(j,i)  * tmMrg_L( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmMrg_L(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmMrg_L( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmMrg_L(M+1,M+1);
              
    mMrg_F(i,j) =      stay(i,j)  *    stay(j,i)  * tmMrg_F( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmMrg_F(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmMrg_F( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmMrg_F(M+1,M+1);
              
    mMrg_A(i,j) =      stay(i,j)  *    stay(j,i)  * tmMrg_A( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmMrg_A(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmMrg_A( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmMrg_A(M+1,M+1);
              
    mPrC_L(i,j) =      stay(i,j)  *    stay(j,i)  * tmPrC_L( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmPrC_L(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmPrC_L( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmPrC_L(M+1,M+1);
              
    mPrC_F(i,j) =      stay(i,j)  *    stay(j,i)  * tmPrC_F( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmPrC_F(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmPrC_F( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmPrC_F(M+1,M+1);
              
    mPrC_A(i,j) =      stay(i,j)  *    stay(j,i)  * tmPrC_A( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmPrC_A(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmPrC_A( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmPrC_A(M+1,M+1);
    if j<i 
        mHerfP(j,i)=mHerfP(i,j);
        mMrg_L(j,i) = mMrg_L(i,j); mMrg_F(j,i)=mMrg_F(i,j); 
        mMrg_A(j,i) = mMrg_A(i,j); 
        mPrC_L(j,i) = mPrC_L(i,j); mPrC_F(j,i)=mPrC_F(i,j); 
        mPrC_A(j,i) = mPrC_A(i,j);
    end
end % j
end % i

% conditioning matrices
mDuopoly = zeros(M+1); mDuopoly(1:M,1:M)=1;
mNonEmpty= ones(M+1); mNonEmpty(M+1,M+1)=0;

% compute expected stats for given distr.
for k=1:K
    % get conditioning probs
    PrDuopoly= sum(sum( mDuopoly .* Distr0(:,:,k) ));
    PrNonEmpty= sum(sum( mNonEmpty .* Distr0(:,:,k) ));
    
    Stats(k).HerfP = sum(sum( mHerfP .* Distr0(:,:,k) ));
    
    Stats(k).PrC(1) = sum(sum( mPrC_L .* Distr0(:,:,k) ));
    Stats(k).PrC(2) = sum(sum( mPrC_F .* Distr0(:,:,k) )) /PrDuopoly;
    Stats(k).PrC(3) = sum(sum( mPrC_A .* Distr0(:,:,k) )) /PrNonEmpty;
    
    Stats(k).Mrg(1) = sum(sum( mMrg_L .* Distr0(:,:,k) ));
    Stats(k).Mrg(2) = sum(sum( mMrg_F .* Distr0(:,:,k) )) /PrDuopoly;
    Stats(k).Mrg(3) = sum(sum( mMrg_A .* Distr0(:,:,k) )) /PrNonEmpty;
end

