% 2 firms, E&E, Symmetry (Herfindahls): timepath and measure-by-state plots
% EE_symmetry(plotTimepath)
%   plotTimepath = 0 - just calculate for limiting distribution
%         = 1 - plot Timepath (& by-state measures, if plotByState below is set

% Angle is disabled due to uncertain origin - (1,1), or (0,0)

function z = EE_symmetry(plotTimepath)
z=0;

% globals
global M % max level of efficiency
global P Pe % transition matix
global DL % Limiting distr., 
global p V stay
% own globals
global lAngl lAngl2 lHerfP lHerfP0 lHerfV % symmetry measures for limiting distribution
global lMode lMean % mode and mean (expectation) of "firm-size" distribution
global PlotHead PlotHead2 PlotDate PlotPage% titles of plot
global maxHerfP amaxHerfP % maximum 
global tHerfP tHerfP0
global i0 j0 maxT state0 

%%% Control variables
%maxT = 100; %i0 = 1; j0 = 1; % - now defined defined as global
plotByState = 0; % plot measures by state
ratePT = 0; % {0} Refreash rate for Timepath plot
waitAK = 0; % {0} 1 to wait for anykey after each timapath plot refresh  

% Transition matrix. (only necessary when plotting - P is not saved)
if isempty(Pe)
    global p stay
    run TransMatEE_duo
end


disp('Calculating symmetry measures ...');

%%% Pre-calculate symmetry measures by competition state (after exit/entry)
tmHerfP = zeros(M+1); 
tmHerfP0 = zeros(M+1); tmHerfP0(M+1,M+1) = 1; 
tmHerfV = ones(M+1); tmHerfV(M+1,M+1) = 0.5; 

for i=1:M+1
for j=1:i
        
    % Herfindahl based on market shares
    F = Fsale( p(i,j), p(j,i) );  % precalculate probabilities
    F0 = Fsale0( p(i,j), p(j,i) );
    
    if i+j < 2*M+2 % i<M+1 or j<M+1, non-empty industry 
        tmHerfP(i,j) = ( F^2 + (1-F-F0)^2 ) / ((1-F0)^2); tmHerfP(j,i) = tmHerfP(i,j);
        tmHerfP0(i,j) = F0^2 + F^2 + (1-F-F0)^2 ; tmHerfP0(j,i) = tmHerfP0(i,j);
    else % i=j=M+1, empty industry
        tmHerfP(i,j) = 0.5; % minimum
    end

    % Herfindahl based on Values
    if ( i < M+1 ) & ( j < M+1 )
        tmHerfV(i,j) = ( V(i,j)^2 + V(j,i)^2 ) / ( V(i,j) + V(j,i) )^2;
        tmHerfV(j,i) = tmHerfV(i,j);
    end
    
end % j
end % i

% Herfs by initial state 
mHerfP = zeros(M+1); 
mHerfP0 = zeros(M+1);  
mHerfV = ones(M+1);  

for i=1:M+1
for j=1:i

   if i<M+1
       ip=i;
   else
       ip=1;
   end
   if j<M+1
       jp=j;
   else
       jp=1;
   end
    
    mHerfP(i,j) =      stay(i,j)  *    stay(j,i)  * tmHerfP( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmHerfP(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmHerfP( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmHerfP(M+1,M+1);
    mHerfP(j,i)=mHerfP(i,j);
    
    mHerfP0(i,j) =      stay(i,j)  *    stay(j,i)  * tmHerfP0( ip, jp) ... 
                   + (1-stay(i,j)) *    stay(j,i)  * tmHerfP0(M+1, jp) ... 
                   +    stay(i,j)  * (1-stay(j,i)) * tmHerfP0( ip,M+1) ... 
                   + (1-stay(i,j)) * (1-stay(j,i)) * tmHerfP0(M+1,M+1);
    mHerfP0(j,i) = mHerfP0(i,j);
    
    mHerfV(i,j) =      stay(i,j)  *    stay(j,i)  * tmHerfV( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * tmHerfV(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * tmHerfV( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * tmHerfV(M+1,M+1);
    mHerfV(j,i) = mHerfV(i,j);
    
end % j
end % i

% mAngl2=mAngl.^2;

%%% Calc measures for limiting distribution
%lAngl = sum(sum( mAngl .* DL ));
%lAngl2 = sqrt( sum(sum( mAngl2 .* DL )) - lAngl.^2 );
lHerfP = sum(sum( mHerfP .* DL ));
lHerfP0 = sum(sum( mHerfP0 .* DL ));
lHerfV = sum(sum( mHerfV .* DL ));

%%% Main plot
if plotTimepath 
    figure('Name','HERFINDAHLs', 'NumberTitle','off');
    orient landscape; 
    %hhh = fheader('');
    if waitAK
        hf = ffooter(sprintf('Press Key to step %d period(s), click mouse to go until end (%d)...', ratePT, maxT));    
    else
        hf = ffooter('');
    end

    % limiting distribution measures
    h = subplot(2,3,5); 
    set(h,'Visible','off');
    ht=title('');
    set(ht,'Visible','on', 'Units','Normal', 'Position',[.5 .3]);
    set(ht,'HorizontalAlignment', 'center');
    TextLim = sprintf(['Limiting distribution: \n' ...
                   'Unconditional Herf.: %f\nMarket share Herf.:%f \nValue Herf.: %f\n\n' ...
                   '  Limiting distribution: \n Mode (%3.1f,%3.1f)'], ... %\n Mean (%3.1f,%3.1f)
                    lHerfP0, lHerfP, lHerfV, lMode(1), lMode(2) );
                %'Expected angle: %f \nSt.Dev. of angle: %f \n', lAngl, lAngl2, 
                
    % top left - Uncond. herfindahl
    h=subplot(2,3,1);   
    mesh([0:M],[0:M], [ mHerfP(M+1,M+1) mHerfP(M+1,1:M); mHerfP(1:M,M+1) mHerfP(1:M,1:M) ]');
    v = axis; v(1) = 0; v(2) = M; v(3) = 0; v(4) = M; v(5)=0.5; v(6)=1; axis(v); % axis: minx,maxx, miny, maxy
    xlabel('e_{1}'); ylabel('e{1}'); set(h, 'YDir','reverse', 'XDir','reverse');
    title( 'Cond-l Mkt. Share Herf.', 'FontSize',11);
            
end % if plotTimepath
                
% init timepath loop
a = zeros(1,(M+1).^2); % Initial distribution (as vector)
a( (i0-1).*(M+1) + j0 ) = 1;
am=zeros(M+1); % distributions as matrix
%tAngl=zeros(1,maxT); tAngl2=zeros(1,maxT); % time paths
tHerfP=zeros(1,maxT); tHerfV=zeros(1,maxT);
tHerfP0=zeros(1,maxT);
tMode=zeros(2,maxT); tMean=zeros(2,maxT);
maxHerfP=0; amaxHerf=0;

for t=1:maxT
    % convert to matrix distr
    for n=1:((M+1).^2)
        j = rem(n,M+1); % calc (i,j)(n)
   	    if j==0
      	    i = n./(M+1); j = M+1;
        else
      	    i = (n-j)./(M+1)+1;
        end
        am(i,j) = a(n);
    end % for n
   
    % calc expected symmetry measures
   %tProd(t) = sum(sum( mProd .* am ));
   %tAngl(t) = sum(sum( mAngl .* am ));
   %tAngl2(t) = sqrt( sum(sum( mAngl2 .* am )) - tAngl(t)^2 );
   tHerfP(t) = sum(sum( mHerfP .* am ));
   tHerfP0(t) = sum(sum( mHerfP0 .* am ));
   tHerfV(t) = sum(sum( mHerfV .* am ));
   
   [tMean(1,t) tMean(2,t) tMode(1,t) tMode(2,t)] = meanmode(am);
   if tMode(1,t) == M+1 % switch to zero exit state
       tMode(1,t) = tMode(2,t);
       tMode(2,t) = 0;
       if tMode(1,t) == M+1 % M+1,M+1
           tMode(1,t) = 0;
       end
   end
   

   %disp(sprintf('t=%d (%2.1f, %2.1f) (%2.1f %2.1f)',[t tMean(:,t)' tMode(:,t)']))
   %disp(tAngl2(1,1:t))
   %disp(sprintf('sum=%f, min=%f', sum(sum(am)), min(min(am) )))

   if tHerfP(t)>maxHerfP
       maxHerfP=tHerfP(t);
       amaxHerfP=t;
   end
   
   % plot (if "it is time")
   if (( ~mod(t-1,ratePT) & ratePT ~= 0 ) | t == maxT) & plotTimepath 
        
       t_ax = max(t,5); % for meaningfull axis limits
       
       
       % top mid - Unconditional herfindahl 
       subplot(2,3,2);   
       plot( 1:t, tHerfP0(1:t)); 
       v = axis; v(1) = 1; v(2) = t_ax; v(3)=1/3; v(4)=1; axis(v); 
       xlabel('t'); ylabel('1/3 - 1');
       title('Herfindahl (unconditional)', 'FontSize',11, 'FontWeight','bold');
    
       % top right - Conditional Herfindahls
       subplot(2,3,3);   
       plot( 1:t, [tHerfP(1:t); tHerfV(1:t)]); 
       v = axis; v(1) = 1; v(2) = t_ax; v(3)=0.5; v(4)=1; axis(v); 
       xlabel('t'); ylabel('0.5 - 1');
       legend('Mkt. share (p)', 'Value share',0 );
       title('Herfindahl (cond. on firms)', 'FontSize',11, 'FontWeight','bold');
       
       % bottom left - distribution (T)
       h = subplot(2,3,4); 
       %bar3([1:M],am');
       %lin = stem3( repmat([0:M]',1,M), repmat([1:M],M,1), am');
       lin = stem3( repmat([0:M]',1,M+1), repmat([0:M],M+1,1), ...
             [am(M+1,M+1) am(M+1,1:M); am(1:M,M+1) am(1:M,1:M)]);
        set(lin, 'Marker', 'none', 'LineWidth', 1);   
       xlabel('e_{1}'); ylabel('e_{2}'); zlabel('\pi_{e}');
       v = axis; v(1) = 0; v(2) = M; v(3) = 0; v(4) = M; v(5) = 0; axis(v);
       set(h,'XTick',[1:3:M]); set(h,'YTick',[1:3:M]); set(h,'YDir','normal'); 
       Title(sprintf('Distribution at T=%d',t), 'FontSize',11, 'FontWeight','bold')
          view(-15,30);
       
       % bottom right - mode timepath
       subplot(2,3,6);   
       plot( 1:t, [tMode(1,1:t); tMode(2,1:t)]); 
       v = axis; v(1) = 1; v(2) = t_ax; v(3)=0; v(4)=M+1; axis(v); 
       xlabel('t'); ylabel(sprintf('Efficiency (0-%d)', M));
       legend('Leader', 'Follower',0 );
       title('Mode of transient distr.', 'FontSize',11);
 
       % bottom mid - text
       TextAll = sprintf(['Distribution at at T=%d: \n' ...
                   'Unconditional Herf.: %f\nMarket share Herf.:%f \nValue Herf.: %f\n\n%s'], ...
                    t, tHerfP0(t), tHerfP(t), tHerfV(t), TextLim);
                %'Expected angle: %f\nSt.Dev. of angle: %f \n', tAngl(t), tAngl2(t), 

       set(ht,'String',TextAll);
       
       ip=i0;
       subtitle(sprintf( '%s:  HERFINDAHLs timepath (starts at %s) \n%s\n', PlotHead2, state0, PlotHead )); 
       
       drawnow;
    
       if waitAK
           k = waitforbuttonpress; % k=0 mouse, k=1 keyboard
           if k == 0
               waitAK = 0;
           end
       end
   
   end % if have to plot
   
   % move to next period
   a = a * Pe;
   
end % T

if ~plotTimepath 
    return
end

if PlotPage>0
    fpage(sprintf( '%s\npage %d', PlotDate, PlotPage ));
    PlotPage = PlotPage + 1;
else
    fpage(PlotDate);
end
    

if ~isempty(hf)
    set(hf,'String','');
end
ffooter(sprintf('Max(Cond. mkt. share Herf) = %4.3f, achieved at t = %d', maxHerfP, amaxHerfP));

%%%% by-state plot
if plotByState & plotTimepath 
    figure('Name','Symmetry: measures by state', 'NumberTitle','off');

    % h=subplot(2,2,1);
    % mesh([1:M],[1:M], mAngl');
    % v = axis; v(1) = 1; v(2) = M; v(3) = 1; v(4) = M; axis(v); % axis: minx,maxx, miny, maxy
    % xlabel('i'); ylabel('j'); set(h, 'YDir','reverse', 'XDir','reverse');
    % title( 'Angle', 'FontSize',11, 'FontWeight','bold' );
    
    h=subplot(2,2,2); % uncond. mkt. share Herf
    mesh([0:M],[0:M], [ mHerfP0(M+1,M+1) mHerfP0(M+1,1:M); mHerfP0(1:M,M+1) mHerfP0(1:M,1:M) ]');
    v = axis; v(1) = 0; v(2) = M; v(3) = 0; v(4) = M; v(5)=1/3; v(6)=1; axis(v); % axis: minx,maxx, miny, maxy
    xlabel('e_{1}'); ylabel('e_{2}'); set(h, 'YDir','reverse', 'XDir','reverse');
    title( 'Market share Herfindahl (firm + alt.) ', 'FontSize',11, 'FontWeight','bold' );
    
    h=subplot(2,2,3); % mkt. share Herf
    mesh([0:M],[0:M], [ mHerfP(M+1,M+1) mHerfP(M+1,1:M); mHerfP(1:M,M+1) mHerfP(1:M,1:M) ]');
    v = axis; v(1) = 0; v(2) = M; v(3) = 0; v(4) = M; v(5)=0.5; v(6)=1; axis(v); % axis: minx,maxx, miny, maxy
    xlabel('e_{1}'); ylabel('e_{2}'); set(h, 'YDir','reverse', 'XDir','reverse');
    title( 'Market share Herf. (cond. on firms)', 'FontSize',11, 'FontWeight','bold' );
    
    h=subplot(2,2,4); % Value Herf
    mesh([0:M],[0:M], [ mHerfV(M+1,M+1) mHerfV(M+1,1:M); mHerfV(1:M,M+1) mHerfV(1:M,1:M) ]'); 
    v = axis; v(1) = 0; v(2) = M; v(3) = 0; v(4) = M; v(5)=0.5; v(6)=1; axis(v); % axis: minx,maxx, miny, maxy
    xlabel('e_{1}'); ylabel('e_{2}'); set(h, 'YDir','reverse', 'XDir','reverse');
    title( 'Herfindahl (value shares)', 'FontSize',11, 'FontWeight','bold' );
    
    subtitle(sprintf( '%s:  SYMMETRY measures by state \n %s\n', PlotHead2, PlotHead));
    if PlotPage>0
        fpage(sprintf( '%s\npage %d', PlotDate, PlotPage ));
        PlotPage = PlotPage + 1;
    else
        fpage(PlotDate);
    end
    
    orient landscape; 
end % if plotByState
