% Expected Welfare (follows Capps-Dranove Satterthwaite)
% 2 firms, Entry and Exit, (Alternative does not matter)
% welfare(plotTimepath,calcP)
%   plotTimepath = 0 - just calculate for limiting distribution
%         = 1 - plot Timepath (& by-state measures, if plotByState below is set
%   calcP = 0 - skip calculating transition matrix (to save time)

function z = EE_welfare(plotTimepath)
z=0;

% Welfare = consumer surplus
% Surplus = total surplus

% globals
global M % max level of efficiency
global Pe % transition matix
global DL % Limiting distr., 
global p V stay
global vOut sigma beta
global v_alt c_alt v_gd % new: utility of alt, cost(=price) of alternative, utility of good
global kappa
global X a S b %entry/exit values 
global XC SS % Exit Cushion and Set-up Subsidy
global PlotHead PlotHead2 PlotDate PlotPage% titles of plot
% 'own' globals, calculated in the script
global lWelfare lSurplus lWelfareD lSurplusD 
    % expected consumer, total surplus, plus discounted (PV)
global lDyears % number of years used to calc discounted surplus
global ProbC Profit Profits
global mSurplus mWelfare
global tWelfare tSurplus
global i0 j0 maxT state0

%%% Control variables
% maxT = 100; i0 = 1; j0 = 1; % now deifned as global
% plotByState = 0; % plot measures by state
ratePT = 0; % {0} Refreash rate: Timepath plot 
waitAK = 0; % {0} 1 to wait for anykey after each plot refresh  

% Transition matrix. (only necessary when plotting - P is not saved)
if isempty(Pe)
    global p stay
    run TransMatEE_duo
end


disp('Calculating welfare measures ...');

%%% Pre-calculate by-state measures
%mWelfare = -vOut + sigma*log( exp(vOut/sigma) + exp(-p/sigma) + exp(-p'/sigma) );
mWelf = v_alt - c_alt + sigma*log( 1 + exp((-vOut-p)/sigma) + exp((-vOut-p')/sigma) );

mSurplus = zeros(M+1); mWelfare = zeros(M+1);
for i=1:M+1
for j=1:i
    % ip, jp - efficiency levels if firm chooses to o(p)erate
    %       i' in the SK note
    
   if i<M+1
       ip=i;
   else
       ip=1;
   end
   if j<M+1
       jp=j;
   else
       jp=1;
   end
    
   %%% surpluses from price competition
       %mWelf is symmetric - adds up surpluses of bying from both firms   
    mWelfare(i,j) =    stay(i,j)  *    stay(j,i)  * mWelf( ip, jp) ... 
                  + (1-stay(i,j)) *    stay(j,i)  * mWelf(M+1, jp) ... 
                  +    stay(i,j)  * (1-stay(j,i)) * mWelf( ip,M+1) ... 
                  + (1-stay(i,j)) * (1-stay(j,i)) * mWelf(M+1,M+1);
                    
       %Profit is not symmetric - profit for one firm only   
    Producer =    stay(i,j)  *    stay(j,i)  * ( Profit( ip, jp) + Profit( jp, ip) ) ... 
             + (1-stay(i,j)) *    stay(j,i)  * Profit( jp,M+1) ... 
             +    stay(i,j)  * (1-stay(j,i)) * Profit( ip,M+1) ... 
             + (1-stay(i,j)) * (1-stay(j,i)) * Profit(M+1,M+1);

    %%% correction to surplus from exit/entry values and subsidy/cushion
    if i<M+1
             Producer =      Producer + (1-stay(i,j)) * ( X + a * stay(i,j) );
        mWelfare(i,j) = mWelfare(i,j) - (1-stay(i,j)) *   XC;
    else
             Producer =      Producer - stay(i,j) * ( S + b * (1-stay(i,j)) );
        mWelfare(i,j) = mWelfare(i,j) - stay(i,j) *   SS;
    end
    
    if j<M+1
             Producer =      Producer + (1-stay(j,i)) * ( X + a * stay(j,i) );
        mWelfare(i,j) = mWelfare(i,j) - (1-stay(j,i)) *   XC;
    else
             Producer =      Producer - stay(j,i) * ( S + b * (1-stay(j,i)) );
        mWelfare(i,j) = mWelfare(i,j) - stay(j,i) *   SS;
    end
    
    % finalize calculation
    mWelfare(j,i)=mWelfare(i,j);              
    mSurplus(i,j) = mWelfare(i,j) + Producer;
    mSurplus(j,i) = mSurplus(i,j);
end % j
end % i


%%% Calc measures for limiting distribution
lWelfare = sum(sum( mWelfare .* DL ));
lSurplus = sum(sum( mSurplus .* DL ));

% mode and mean
   
% check if Timepath plot is needed
if plotTimepath 

    %%% Main plot
    figure('Name','Surplus time path', 'NumberTitle','off');
    %hhh = fheader('');
    if waitAK
        hf = ffooter(sprintf('Press Key to step %d period(s), click mouse to go until end (%d)...', ratePT, maxT));    
    else
        hf = [];
    end
    
    
        % bottom mide: limiting distribution measures (text)
    h = subplot(2,3,5); 
    set(h,'Visible','off');
    ht=title('');
    set(ht,'Visible','on', 'Units','Normal', 'Position',[.5 .3]);
    set(ht,'HorizontalAlignment', 'center');
    TextLim = sprintf(['Limiting distribution:\n '...
            'Expected: %1.2f + %1.2f = %1.2f'], ...
         lWelfare, lSurplus-lWelfare, lSurplus );

    % top left - by-state welfare
    subplot(2,3,3);
    mesh([0:M],[0:M],[ mWelfare(M+1,M+1) mWelfare(M+1,1:M); mWelfare(1:M,M+1) mWelfare(1:M,1:M)]');
    v = axis; v(1) = 0; v(2) = M; v(3) = 0; v(4) = M; axis(v); % axis: minx,maxx, miny, maxy
    %mesh([1:M],[1:M], [mWelfare]');
    %v = axis; v(1) = 1; v(2) = M; v(3) = 1; v(4) = M; axis(v); % axis: minx,maxx, miny, maxy
    xlabel('e_{1}'); ylabel('e_{2}');
    title( 'Consumer surplus by state', 'FontSize',11);
    
    % bottom left - by-state surplus
    subplot(2,3,6);
    mesh([0:M],[0:M],[ mSurplus(M+1,M+1) mSurplus(M+1,1:M); mSurplus(1:M,M+1) mSurplus(1:M,1:M)]');
    v = axis; v(1) = 0; v(2) = M; v(3) = 0; v(4) = M; axis(v); % axis: minx,maxx, miny, maxy
    %mesh([1:M],[1:M], mSurplus');
    %v = axis; v(1) = 1; v(2) = M; v(3) = 1; v(4) = M; axis(v); % axis: minx,maxx, miny, maxy
    xlabel('e_{1}'); ylabel('e_{2}');
    title( 'Total surplus by state', 'FontSize',11);
end % if plotTimepath 
    
% init timepath loop
vtd = zeros(1,(M+1).^2); % Initial distribution (as vector)
vtd( (i0-1).*(M+1) + j0 ) = 1;
mtd=zeros(M+1); % distributions as matrix
tWelfare = zeros(1,maxT); tSurplus = zeros(1,maxT);
tWelfareD = zeros(1,maxT); tSurplusD = zeros(1,maxT);
tWelfareA = zeros(1,maxT); tSurplusA = zeros(1,maxT); % averages
tMode=zeros(2,maxT); tMean=zeros(2,maxT);

for t=1:maxT
    % convert to matrix distr
    for n=1:((M+1).^2)
        j = rem(n,M+1); % calc (i,j)(n)
   	    if j==0
      	    i = n./(M+1); j = (M+1);
        else
      	    i = (n-j)./(M+1)+1;
        end
        mtd(i,j) = vtd(n);
    end % for n
   
    % calc results
   tWelfare(t) = sum(sum( mWelfare .* mtd ));
   tSurplus(t) = sum(sum( mSurplus .* mtd ));
   if plotTimepath
       [tMean(1,t) tMean(2,t) tMode(1,t) tMode(2,t)] = meanmode(mtd);
       if tMode(1,t) == M+1 % switch to zero exit state
          tMode(1,t) = tMode(2,t);
          tMode(2,t) = 0;
       end
   end % if not, no need to waste time calculating
       
   if t>1
       tWelfareD(t) = beta^t * tWelfare(t) + tWelfareD(t-1);
       tSurplusD(t) = beta^t * tSurplus(t) + tSurplusD(t-1);
   else
       tWelfareD(t) = beta * tWelfare(t);
       tSurplusD(t) = beta * tSurplus(t);
   end
   
   tWelfareA(t) = sum(tWelfare)/t;
   tSurplusA(t) = sum(tSurplus)/t;
   
   if t==M & plotTimepath% capture distribution before it hits edges
      TextLim = sprintf(['T=%d (M): \nExpected: %1.2f + %1.2f = %1.2f \n' ...
                        'Discounted: %1.2f + %1.2f = %1.2f\n\n%s'], ...
                        t, tWelfare(t),tSurplus(t)-tWelfare(t), tSurplus(t), ...
                        tWelfareD(t),tSurplusD(t)-tWelfareD(t),tSurplusD(t), TextLim);

   end
 
   % plot (if "it is time")
   if (( ~mod(t-1,ratePT) & ratePT ~= 0 ) | t == maxT) & plotTimepath 
        
       t_ax = max(t,5); % for meaningfull axis limits
       
       % top left - Levels
       subplot(2,3,1);   
       plot( [1:t],tWelfare(1:t),'-b', [1:t],tWelfareA(1:t),':b', ...
             [1:t], tSurplus(1:t),'-g', [1:t], tSurplusA(1:t),':g' ); 
       v = axis; v(1) = 1; v(2) = t_ax; axis(v); %v(3)=0; v(4)=0.75; 
       xlabel('t'); ylabel('Utility ($)');
       legend('Consumer surplus','   its average', 'Total surplus','   its average', 0);
       title('Level and running average', 'FontSize',11, 'FontWeight','bold' );
    
       % mid left - Discounted
       subplot(2,3,2);   
       plot( [1:t], [tWelfareD(1:t);tSurplusD(1:t)]); 
       v = axis; v(1) = 1; v(2) = t_ax; axis(v); %v(3)=0; v(4)=0.75; 
       xlabel('t'); ylabel('Utility ($)');
       legend('Consumer surplus','Total surplus', 0);
       title('Discounted (NPV)', 'FontSize',11, 'FontWeight','bold' );
    
       % bottom left - distribution (T)
       h = subplot(2,3,4); 
       plot( 1:t, [tMode(1,1:t); tMode(2,1:t)]); 
       v = axis; v(1) = 1; v(2) = t_ax; v(3)=0; v(4)=M+1; axis(v); 
       xlabel('t'); ylabel('Efficiency (1-20)');
       legend('Leader', 'Follower',0 );
       title('Mode of transient distr.', 'FontSize',11);
       
       % bottom mid - text
       TextAll = sprintf(['VALUES (cons. + prod. = total)\n\n' ...
                        'T=%d: \nExpected: %1.2f + %1.2f = %1.2f \n' ...
                                ' Average: %1.2f + %1.2f = %1.2f \n' ...
                        'Discounted: %1.2f + %1.2f = %1.2f \n\n%s'], ...
                        t, tWelfare(t),tSurplus(t)-tWelfare(t),tSurplus(t), ...
                        tWelfareA(t),tSurplusA(t)-tWelfareA(t),tSurplusA(t), ...
                        tWelfareD(t),tSurplusD(t)-tWelfareD(t),tSurplusD(t), TextLim);

       set(ht,'String',TextAll);
       
       % generate subsidy/cusion header
       SubCush='';
       if XC~=0
           SubCush=[SubCush sprintf(', Cusion = %1.1f', XC)];
       end
       if SS~=0
           SubCush=[SubCush sprintf(', Subsidy = %1.1f', SS)];
       end
           
       subtitle(sprintf( '%s%s:  SURPLUS timepath (start at %s)\n%s\n', PlotHead2, SubCush, state0, PlotHead )); 
        %set(hhh,'String',sprintf( 'Symmetry Timepath for T=%d  %s\n', t, PlotHead ));


       drawnow;
    
       if waitAK
           k = waitforbuttonpress; % k=0 mouse, k=1 keyboard
           if k == 0
               waitAK = 0;
           end
       end
   
   end % if RateP
   
   % move to next period
   vtd = vtd * Pe;
   
end % T

if plotTimepath % finalize the graphs
    if PlotPage>0
        fpage(sprintf( '%s\npage %d', PlotDate, PlotPage ));
        PlotPage = PlotPage + 1;
    else
        fpage(PlotDate);
    end
    drawnow;

    set(hf,'String','');
    orient landscape; 
    
end

lDyears = maxT;
lWelfareD = tWelfareD(maxT);
lSurplusD = tSurplusD(maxT);

