// FOC for duopoly problem

// include libraries
#include "mex.h" // links to matlab
#include <math.h> // math functions (exp)

double beta, v0, sigma;  /// declare global variables

       // main function
double FOC(double p1, double p2, double c, double wa, double w, double l)
{
    double F, F0, z;

    // pre-calculate probabilites (no need for separate f-n)
    F0 =  1 /( 1 + exp( (-p1 - v0)/ sigma )
                 + exp( (-p2 - v0)/ sigma ) );

    F  = 1 / (      exp( (p1 + v0)/ sigma )
              + 1 + exp( (p1 - p2)/ sigma ) );

    return sigma - (1-F) * ( p1 - c + beta * w )
        + beta * ( F0 * wa + (1 - F - F0) * l) ;
}

// "Gateway function" - converts from Matlab to C vars
// also "entry point" - runs when .dll is called
void mexFunction(int nlhs, mxArray *plhs[], int nrhs,
                 const mxArray *prhs[])
{
	double p1, p2, c, wa, w, l, *z;

    if (nrhs != 6)
        mexErrMsgTxt("FOC_ac: Six inputs required.");

    // read inputs
	p1 = *mxGetPr(prhs[0]); // 1st argument
    p2 = *mxGetPr(prhs[1]);
    c  = *mxGetPr(prhs[2]);
    wa = *mxGetPr(prhs[3]);
    w  = *mxGetPr(prhs[4]);
    l  = *mxGetPr(prhs[5]);

    // init output
    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    z = mxGetPr(plhs[0]);

    // read paramenter
	beta = *mxGetPr(mexGetVariablePtr("global", "beta"));
    v0 = *mxGetPr(mexGetVariablePtr("global", "vOut"));
    sigma = *mxGetPr(mexGetVariablePtr("global", "sigma"));

	z[0] = FOC(p1, p2, c, wa, w, l); // run main function

}
