CCC	Numeric dense (full) Jacobian - one colum at a time
C
C Return in the vector  V  the Kth column of the Jacobian
C matrix [D RHO/D LAMBDA, D RHO/DX] evaluated at the point
C (A, LAMBDA, X).
C
C	two-sided finite difference scheme	
C
      SUBROUTINE RHOJAC(A,LAMBDA,X,V,K)
      USE REAL_PRECISION, ONLY : R8
C      USE HOMPACK90_GLOBAL, ONLY: IPAR, PAR  ! for POLSYS1H only.
      USE HOMPACK90_GLOBAL
      REAL (KIND=R8), INTENT(IN):: A(:),X(:)
      REAL (KIND=R8), INTENT(IN OUT):: LAMBDA
      REAL (KIND=R8), INTENT(OUT):: V(:)
      INTEGER, INTENT(IN):: K
C	variables for fintie difference computation
	DOUBLE PRECISION HL(size(X)), HR(size(X)), X2(size(X)) 
	REAL (KIND=R8), PARAMETER:: DELTA = 1.0D-7
C
C	other variables
      INTEGER:: N, NP1 ! size of X
      INTEGER:: I ! loop index
C	
	INTERFACE
	  SUBROUTINE RHO (A,LAMBDA,X,H)
          USE REAL_PRECISION, ONLY : R8
		REAL (KIND=R8), INTENT(IN) :: A(:),X(:)
		REAL (KIND=R8), INTENT(IN OUT) :: LAMBDA
		REAL (KIND=R8), INTENT(OUT) :: H(:)
	  END SUBROUTINE RHO
	END INTERFACE
C
C	initialize and reset
	N = size(X)
	NP1=N+1
	HL = 0.0D0
	HR = 0.0D0
	X2(1:N)=X(1:N)
C
C
C	left value
	IF (K == 1) THEN 
		LAMBDA = LAMBDA - DELTA/2.0
	ELSE
		X2(K-1) = X2(K-1) - DELTA/2.0
	END IF
	CALL RHO(A,LAMBDA,X2,HL) 

C		right value
	IF (K == 1) THEN 
		LAMBDA = LAMBDA + DELTA
	ELSE
		X2(K-1) = X2(K-1) + DELTA
	END IF
	CALL RHO(A,LAMBDA,X2,HR) 

	DO 20 I=1,N
		IF (HR(I) /= HL(I))  THEN
			V(I) = (HR(I) - HL(I))/DELTA
		ELSE
			V(I) = 0.0D0
		END IF
 20	CONTINUE	! I=1,N
	RETURN
      END
