      SUBROUTINE RHOJS(A,LAMBDA,X)
C
C Subroutine RHOJS(A,LAMBDA,X) computes the Jacobian matrix of
C rho(a,x,lambda) at the point (A,X,LAMBDA), 
C returning the Jacobian matrix in sparse row
C storage format (MODE = 2) in the arrays QRSPARSE, ROWPOS, and
C COLPOS.
C	SK: Specifically, Nx(N+1) jacobian of rho(x,lambda)
C		columns 1:N = dRho/dx, column N+1 = dRho/dlambda
C  QRSPARSE - Jacobian elements themselves, in row order
C  LENQR is the length of QRSPARSE 
C	SK: only used to create it, not in this routine
C  ROWPOS contains the indices of where each row begins within QRSPARSE, and 
C	SK: last element of ROWPOS contains LENQR+1
C  COLPOS (of length LENQR) contains the column indices of
C    the corresponding elements in QRSPARSE.  
C Even if zero, the diagonal elements of the Jacobian matrix 
C         must be stored in QRSPARSE.
C
C	Implementation: ADIFOR's augmented routine gives columns, 
C	which are stored into pre-defined sparsity structure (ROWPOS, COLPOS)
C	if numeric differentiation gives a NZ that's not part of the structure, error is generated. 

      USE REAL_PRECISION, ONLY : R8
      USE HOMPACK90_GLOBAL, ONLY: QRSPARSE, ROWPOS, COLPOS
      USE SWITCH
      REAL (KIND=R8), INTENT(IN OUT):: A(:),LAMBDA,X(:)

C	variables for Adifor derivative calculation
	DOUBLE PRECISION Y(size(X)+1), DY(size(X)+1), DH(size(X)), 
     &			H(size(X))
      INTEGER K ! column

C	other variables
      INTEGER:: N, NP1 ! size of X
      INTEGER:: I, IK, QPOS, RPOS ! various array indeces
C      REAL (KIND=R8):: DRHODL(size(X))
C
      INTERFACE
      subroutine g_rho1(n, x, g_x, h, g_h)
        integer n
        double precision x(n + 1), h(n)
        double precision g_x(n + 1), g_h(n)
	end subroutine g_rho1
	END INTERFACE
C
C	initialize and reset
	N = size(X)
	NP1=N+1
	QRSPARSE=0.0
	DH = 0.0
	DY = 0.0
	H = 0.0

	Y(1:N)=X(1:N)
	Y(N+1)=LAMBDA

!	write(*,'(A,I5,A,E11.2)') '   RHOJS started,N=',N,
!     &            ', lbd=',LAMBDA

      DO 100 K=1,NP1


C		compute dH/dY(K)
		DY(K)=1.0
		call g_rho1(N, Y,DY, H,DH)

C		record non-zero derivatives
!		write(*,'(A,I5)'), 'Jacobian column ',K
		DO 20 I=1,N
			IF ((DH(I) /= 0).OR.(I==K))  THEN
C				find position of (I,K) in QRSPARSE
				QPOS = ROWPOS(I) ! element inside row
				DO 30 WHILE ( (COLPOS(QPOS)/=K)
     &				        .AND.(QPOS<ROWPOS(I+1)) )
 30				QPOS=QPOS+1
 	
				if(QPOS>=ROWPOS(I+1)) then
					write(*,'(A,I4,A,I4,A)' ),  
     &				 'Unexpected nonzero in Jacobian in (',I,
     &				 ',',K,')'
					stop
				end if
				
				QRSPARSE(QPOS) = DH(I)
				
			END IF
!			if ( (V(J)/=0) .and. (K>1) ) then
!				write(*, '(A,I5,A,F10.4)') ' row ',J,', value ',V(J)
!			end if
 20		CONTINUE	! I=1,NEQ
	
C	  return to original state	
		DY(K)=0.0
C
 100	CONTINUE ! K=1,NP1 - numeric Jacobian

!	print *, '   Sparce Numeric Jacobian computed'

      RETURN
      END
