      SUBROUTINE RHOJS(A,LAMBDA,X)
C
C Subroutine RHOJS(A,LAMBDA,X) computes the Jacobian matrix of
C rho(a,x,lambda) at the point (A,X,LAMBDA), 
C returning the Jacobian matrix in sparse row
C storage format (MODE = 2) in the arrays QRSPARSE, ROWPOS, and
C COLPOS.
C	SK: Specifically, Nx(N+1) jacobian of rho(x,lambda)
C		columns 1:N = dRho/dx, column N+1 = dRho/dlambda
C  QRSPARSE - Jacobian elements themselves, in row order
C  LENQR is the length of QRSPARSE 
C	SK: only used to create it, not in this routine
C  ROWPOS contains the indices of where each row begins within QRSPARSE, and 
C	SK: last element of ROWPOS contains LENQR+1
C  COLPOS (of length LENQR) contains the column indices of
C    the corresponding elements in QRSPARSE.  
C Even if zero, the diagonal elements of the Jacobian matrix 
C         must be stored in QRSPARSE.
C
C	Implementation: numeric differentiation gives columns, 
C	which are stored into pre-defined sparsity structure (ROWPOS, COLPOS)
C	if numeric differentiation gives a NZ that's not part of the structure, error is generated. 

      USE REAL_PRECISION, ONLY : R8
      USE HOMPACK90_GLOBAL, ONLY: QRSPARSE, ROWPOS, COLPOS
      USE SWITCH
      REAL (KIND=R8), INTENT(IN OUT):: A(:),LAMBDA,X(:)

C	variables for Numeric derivative calculation
	REAL (KIND=R8), PARAMETER:: DELTA = 1.0D-7
	REAL (KIND=R8):: HL(size(X)),HR(size(X))
      INTEGER K ! column
	integer:: MaxZeros ! max number of zeros per Jacobian row
	COMMON/MAXZERO/MaxZeros

C	other variables
      INTEGER:: N, NP1 ! size of X
      INTEGER:: I, IK, QPOS, RPOS ! various array indeces
C      REAL (KIND=R8):: DRHODL(size(X))
C
      INTERFACE
	  SUBROUTINE RHO (A,LAMBDA,X,H)
          USE REAL_PRECISION, ONLY : R8
		REAL (KIND=R8), INTENT(IN) :: A(:),X(:)
		REAL (KIND=R8), INTENT(IN OUT) :: LAMBDA
		REAL (KIND=R8), INTENT(OUT) :: H(:)
	  END SUBROUTINE RHO
      END INTERFACE
C
C	initialize and reset
	N = size(X)
	NP1=N+1
	QRSPARSE=0.0
	
C	**** Stage 1. Numeric Jacobian ************
C	for each row, there are MaxZeros elements in QRSPARSE & COLPOS
C	first a solid block of non-zeros, then zeros in both QRSPARSE & COLPOS
      DO 100 K=1,NP1

C		left value
		IF (K == NP1) THEN 
			LAMBDA = LAMBDA - DELTA/2.0
		ELSE
			X(K) = X(K) - DELTA/2.0
		END IF
		CALL RHO(A,LAMBDA,X,HL) 

C		right value
		IF (K == NP1) THEN 
			LAMBDA = LAMBDA + DELTA
		ELSE
			X(K) = X(K) + DELTA
		END IF
		CALL RHO(A,LAMBDA,X,HR) 

C		compute and record non-zero derivatives
!		write(*,'(A,I5)'), 'Jacobian column ',K
		DO 20 I=1,N
			IF (HR(I) /= HL(I).OR.(I==K))  THEN
C				position of (I,K)
				QPOS = ROWPOS(I) ! element inside row
				DO 30 WHILE ( (COLPOS(QPOS)/=K)
     &				        .AND.(QPOS<ROWPOS(I+1)) )
 30				QPOS=QPOS+1
 	
				if(QPOS>=ROWPOS(I+1)) then
					write(*,'(A,I4,A,I4,A)' ),  
     &				 'Unexpected nonzero in Jacobian in (',I,
     &				 ',',K,')'
					stop
				endif
				
				IF (COLPOS(QPOS)==K) THEN
					QRSPARSE(QPOS) = (HR(I) - HL(I))/DELTA
				ELSE
					STOP 'QPOS/=K :O'
				END IF
				
			END IF
!			if ( (V(J)/=0) .and. (K>1) ) then
!				write(*, '(A,I5,A,F10.4)') ' row ',J,', value ',V(J)
!			end if
 20		CONTINUE	! I=1,NEQ
	
C	  return to original state	
		IF (K == NP1) THEN 
			LAMBDA = LAMBDA - DELTA/2.0
		ELSE
			X(K) = X(K) - DELTA/2.0
		END IF
C
 100	CONTINUE ! K=1,NP1 - numeric Jacobian

!	print *, '   Sparce Numeric Jacobian computed'

      RETURN
      END
