!	Sparse Jacobian, computed numerically
! read binary file with parameters & X-vector of homotopy, launch it
! based on old EZ_hompack maindriver by Schmedders and HOmpack90's test2.f

	MODULE SWITCH
!  ROWSET  IS USED TO INITIALIZE SPARSE MATRIX DATA STRUCTURES ONLY
!  ONCE, AFTER THEY ARE ALLOCATED.
	LOGICAL::  ROWSET
	END MODULE SWITCH


C **************** start main program *******************
	PROGRAM maindriver

	USE SWITCH
	USE REAL_PRECISION, ONLY : R8
	USE HOMPACK90, ONLY : FIXPNS
	USE HOMPACK90_GLOBAL, ONLY: QRSPARSE, ROWPOS, COLPOS ! only for test

	IMPLICIT NONE
	! INTEGER, PARAMETER:: N=8, NDIMA=8

	integer M0,N0 !! M and size of X as constants for declarations
	parameter (M0=30, N0=M0*M0*2) !! maybe replace with dynamic allocation

	integer ios, fid ! input-output status
	parameter (fid=13) ! file handle

!     model parameters
	integer M, mmax
	real*8 vOut, sigma
	real*8 delta_beg, delta_end
	real*8 beta
	COMMON/ModelParams/beta,vOut,sigma,delta_beg,delta_end,M,mmax 
	real*8 rho_beg, rho_end, kappa
	COMMON/CostParams/rho_beg, rho_end, kappa

! offsets in X and H vectors
	integer offP, offV
	integer offBell, offFOC
	COMMON/OFFSETS/offP,offV,offBell,offFOC

! hompack90 variables 
!	real*8 
	REAL (KIND=R8):: A(N0),X(N0),ANSAE,ANSRE,ARCAE,ARCRE, 
     &    ARCLEN,DTIME,SSPAR(8),Y(N0+1),H(N0)
	INTEGER:: IFLAG,II,LENQR,MODE,NFE,N,NP1,TIMENEW(8),
     &   TIMEOLD(8),TRACE,NDIMA
	character* 100 startFileName

! service variables
	integer i,j, argmaxH
	real*8 maxH, temp(14), lambda

!	function interface
      INTERFACE
	  SUBROUTINE RHO (A,LAMBDA,X,H)
          USE REAL_PRECISION, ONLY : R8
		REAL (KIND=R8), INTENT(IN) :: A(:),X(:)
		REAL (KIND=R8), INTENT(IN OUT) :: LAMBDA
		REAL (KIND=R8), INTENT(OUT) :: H(:)
	  END SUBROUTINE RHO

        SUBROUTINE RHOJS(A,LAMBDA,X)
		USE REAL_PRECISION, ONLY : R8
	    REAL (KIND=R8), INTENT(IN OUT):: A(:),LAMBDA,X(:)
	  END SUBROUTINE RHOJS

	  SUBROUTINE hom_InitializeStepfiles(startFileName)
		character* (*),	INTENT(OUT)::startFileName
	  END SUBROUTINE
      END INTERFACE

C	determine working directory	
	CALL hom_InitializeStepfiles(startFileName)	

C	this is an example of doing all preparation in one subroutine
!	 open file
#ifdef GFORTRAN
	open( UNIT=fid, FILE="hom_start.dat", STATUS="OLD",  
     & ACTION = "READ", IOSTAT = ios, ACCESS = "STREAM")
#else
	open( UNIT=fid, FILE="hom_start.dat", STATUS="OLD",  
     & ACTION = "READ", IOSTAT = ios, FORM = "BINARY")
#endif
	if (ios>0) then
		print *, "ios=", float(ios)
		stop "Fortran: cannot open file for reading"
	else
		print *, "reading starting data"
	endif

!	read parameters
	read(fid) (temp(i),i=1,10)
!	order of parameters: Matlab code
!	c1=fwrite(fid, [M mmax beta kappa vOut sigma ... 
!		   delta2 endDelta rho endRho X a S b], 'float64');
      M=INT(temp(1))
	mmax=INT(temp(2))
      beta = temp(3)
      kappa = temp(4)
      vOut = temp(5)
      sigma = temp(6)
	delta_beg = temp(7)
	delta_end = temp(8)
	rho_beg = temp(9)
	rho_end = temp(10)
	if (M/=M0) stop "Wrong M supplied to fortran"

	read(fid) (Y(i),i=1,N0) 
!	Y(N0+1)=0.0

	close(fid)

!	print *, "   Parameters and X-vector loaded"
! compute offsets in X and H vectors
	offV = 0 
	offP = M*M 
	offBell = 0 
	offFOC = M*M 

	N=N0
	NDIMA=N
!	MaxZeros=16
!	LENQR=MaxZeros*N ! max number of non-zeros in Jacobian (actually, less)
!	DEFINE ARGUMENTS FOR CALL TO HOMPACK PROCEDURE.

	MODE=2
	NP1=N+1
	ARCRE=1.0D-10 ! path (arc) precision, relative and absolut
	ARCAE=1.0D-10
	ANSRE=1.0D-12 ! answer precision, relative and absolut
	ANSAE=1.0D-12
	TRACE=0 ! output file handle, 0 for none
	SSPAR=0.0
	IFLAG=-2 ! -2 for path following - SK
	IF(IFLAG .EQ. -2) A=Y(1:N)
	ROWSET = .FALSE.

!	 *** test run of homotopy function (system of equations)
	X(1:M*M)=Y((M*M+1):N)
	X((M*M+1):N)=Y(1:M*M)
	Y(1:N)=X(1:N)
!	X=Y(1:N)
	H(1:N)=0.0
	lambda=0.0
!	print *, 'testing system of equations (RHO function)'
	call RHO(A,lambda,X,H)

	maxH=0.0
	argmaxH=0
	do 120 i=1,N
	  if (dabs(H(i)).gt.maxH) then
     		maxH=dabs(H(i))
		argmaxH=i
	  endif
 120  continue

	write(*, '(A,E8.3,A,I4)'),'  RHO done, max(abs) = ', maxH,  
     &                           ' ,element # ', argmaxH


C	SETUP SPARSE MATRIX
	CALL QR_STRU(N,LENQR)

	CALL RHOJS(A,LAMBDA,X)
	PRINT *, '  RHOJS done'

	open( UNIT=TRACE, FILE="hom_out.txt", STATUS="REPLACE",  
     & ACTION = "WRITE", IOSTAT = ios, FORM = "FORMATTED")
	if (ios>0) stop "cannot open output file"
	write(*,'(A,F5.3, F6.3,A,F4.2,F5.2,A)'), 
     &     'starting homotopy for delta=[', 
     &    delta_beg, delta_end,'], rho=[',rho_beg, rho_end,']'

	write(*, '(A,I,A,F5.0,A,F5.2,A,F6.3)'),' Starting at m=', mmax,  
     &     ' vOut= ', vOut,'rho=',rho_beg,'delta=', delta_beg

	write(TRACE, '(A,I,A,F5.0,A,F5.2,A,F6.3)'),' Starting at m=', mmax,  
     &     ' vOut= ', vOut,'rho=',rho_beg,'delta=', delta_beg

	
C	**** run the homotopy ****

      CALL FIXPNS(N,Y,IFLAG,ARCRE,ARCAE,ANSRE,ANSAE,TRACE,A,
     &       NFE,ARCLEN,MODE,LENQR,SSPAR)

!      CALL FIXPDS(N,Y,IFLAG,ARCRE,ANSRE,TRACE,A,NDIMA,NFE,
!     &      ARCLEN,MODE,LENQR)
	write(*,'(A,I2)'), 'homotopy (fixPNS) done, flag=', IFLAG
	write(TRACE,'(A,I2)'), 'homotopy (fixPNS) done, flag=', IFLAG

C	**** save Sparse Jacobian
!	open( UNIT=fid, FILE="JacSparse.dat", STATUS="NEW",  
!     & ACTION = "WRITE", IOSTAT = ios, FORM = "BINARY")
!	if (ios>0) then
!		print *, "ios=", float(ios)
!		stop "   cannot create jacobian file (it exists already?)"
!	end if
!	write(fid) (ROWPOS(i),i=1,N+1) 
!	write(fid) (COLPOS(i),i=1,LENQR) 
!	write(fid) (QRSPARSE(i),i=1,LENQR) 
!	close(fid)
!	print *, 'Sparse Jacobian saved'

C	free-up memory
      IF (ALLOCATED(QRSPARSE)) DEALLOCATE(QRSPARSE)
      IF (ALLOCATED(ROWPOS)) DEALLOCATE(ROWPOS)
      IF (ALLOCATED(COLPOS)) DEALLOCATE(COLPOS)

	close(TRACE)
!	PAUSE
	END PROGRAM maindriver

C	********************************************************


! temp. plugs - to be replaced by actual functions in their own files

!      SUBROUTINE RHOJS(A,LAMBDA,X)
!	print *, 'rhojs was called, but not available'
!      RETURN
!      END SUBROUTINE RHOJS

!	SUBROUTINE RHO(A,LAMBDA,X,H)
!	print *, 'rhojs was called, but not available'
!      RETURN
!      END SUBROUTINE RHO

! plugs: subroutines are required for compilation, 
! but never called at runtime
      SUBROUTINE F(X,V)

	print *, 'F called'
      END SUBROUTINE F

      SUBROUTINE FJAC(X,V,K)
	print *, 'FJAC was called'
      RETURN
      END SUBROUTINE FJAC

      SUBROUTINE RHOA(A,LAMBDA,X)
	print *, 'RHOA was called'
      RETURN
      END SUBROUTINE RHOA

      SUBROUTINE FJACS(X)
	print *, 'FJACS was called'
      RETURN
      END SUBROUTINE FJACS

      SUBROUTINE RHOJAC(A,LAMBDA,X,V,K)
	print *, 'RHOJAC was called'
      RETURN
      END SUBROUTINE RHOJAC

