      SUBROUTINE QR_STRU(N,LENQR)
C	defines srarsity structure of Jacobian, in compacted row format
C	this implementation: LBD/OF with Entry and Exit

C	Nx(N+1) jacobian of rho(x,lambda)
C		columns 1:N = dRho/dx, column N+1 = dRho/dlambda
C  QRSPARSE - non-zero elements of Jacobian matrix, in row order
C	withing a row, elements do not have to be ordered
C  LENQR is the length of QRSPARSE, used to allocate memory 
C  ROWPOS contains the indices of where each row begins within QRSPARSE, and 
C  COLPOS (of length LENQR) contains the column indices of
C    the corresponding elements in QRSPARSE.  

C	takes arguments defined in HOMPACK90_GLOBAL
C	N is number of equations (& variables)
C	returns in LENQR the length of QRSPARSE


C	uses SUBROUTINE RECNZ(QPOS,K,LENQR,RR)
C	to records nonzero in column K of row RR
C	it also checks if K is already recorded, and exits if it does
	
      USE HOMPACK90_GLOBAL, ONLY: QRSPARSE, ROWPOS, COLPOS

	INTEGER, INTENT(IN):: N
	INTEGER, INTENT(IN OUT):: LENQR

C	offsets in X and H vectors
	integer offP, offV
	integer offBell, offFOC
	COMMON/OFFSETS/offP,offV,offBell,offFOC

C	model parameters, only needed for M
	integer M, mmax
	real*8 vOut, sigma
	real*8 delta_beg, delta_end
	real*8 beta
	COMMON/ModelParams/beta,vOut,sigma,delta_beg,delta_end,M,mmax 

C	indices & MISC
	INTEGER I,J, I0,I1, J1,J0, I2,J2 ! 2-dimensional state space
	INTEGER QPOS, RR ! position in sparse arrays
	INTEGER NP1 ! N+1

!	average number of zeros per Jacobian row
!	enter maximum if average is unknown
	INTEGER, PARAMETER:: AveZeros=13
	
	IF (N/=M*2*M) STOP "wrong N supplied to QR_STRU"
	NP1=N+1
	
C	ALLOCATE ARRAYS
      IF (ALLOCATED(QRSPARSE)) DEALLOCATE(QRSPARSE)
      IF (ALLOCATED(ROWPOS)) DEALLOCATE(ROWPOS)
      IF (ALLOCATED(COLPOS)) DEALLOCATE(COLPOS)
	LENQR = N*AveZeros
      ALLOCATE(QRSPARSE(LENQR))
      ALLOCATE(ROWPOS(N+2))
      ALLOCATE(COLPOS(LENQR))
	QRSPARSE=0.0
	ROWPOS=0
	COLPOS=0

	QPOS = 1

!	*** start of model-specific code ***
!	must go through rows in their actual order (1,2,3...)

C	**** first set of rows - Bellman, 
C	p,V for under both active and inactive competitors, OperProb for comp. only
	DO 100 J=1,M
	DO 100 I=1,M
		RR = offBell + I+(J-1)*M
		ROWPOS(RR) = QPOS

C		Values
		DO 120 J2=MAX(1,J-1),MIN(J+1,M)
		DO 120 I2=MAX(1,I-1),MIN(I+1,M)
		  IF ( (I2<=I).OR.(J2<=J) ) THEN ! not (I+1,J+1)
			CALL RECNZ(QPOS, offV+ I2+(J2-1)*M, LENQR,RR)  
		  ENDIF
 120		CONTINUE

C		prices		
		CALL RECNZ(QPOS,offP+ I+(J-1)*M,LENQR,RR) ! own p
		CALL RECNZ(QPOS,offP+ J+(I-1)*M,LENQR,RR) ! competitor's p
		
		CALL RECNZ(QPOS,NP1,LENQR,RR) ! homotopy parameter (lambda), last column

 100	CONTINUE ! I,J

!	 **** 2nd set of rows - FOC on p
	DO 200 J=1,M
	DO 200 I=1,M
		RR = offFOC + I+(J-1)*M
		ROWPOS(RR) = QPOS

C		VALUEs 
          DO 220 J2=MAX(1,J-1),MIN(J+1,M)
          DO 220 I2=MAX(1,I-1),MIN(I+1,M)
             IF ( (I2<=I).OR.(J2<=J) ) THEN ! not (I+1,J+1)
                CALL RECNZ(QPOS,offV + I2+(J2-1)*M,LENQR,RR)  
             ENDIF
 220      CONTINUE

C		prices		
          CALL RECNZ(QPOS,offP + J+(I-1)*M,LENQR,RR) ! competitor's price
		CALL RECNZ(QPOS,offP + I+(J-1)*M,LENQR,RR) ! own price

		CALL RECNZ(QPOS,NP1,LENQR,RR) ! homotopy parameter (lambda), last column

 200	CONTINUE ! I,J

!	*** end of model-specific code ***

C	Finalize definitions
	ROWPOS(NP1) = QPOS
	LENQR=QPOS-1	
	WRITE(*, '(A,I6,A,F4.1,A)'), ' sparse Jacobian structure created, '
     &          ,QPOS-1, ' non-zeros, ', 
     &          float(QPOS-1)/float(N), ' per row'

C	SORT ROW INDECES - BUBLE
	DO 400 I=1,N
		DO 440 J1=ROWPOS(I),(ROWPOS(I+1)-2)
		DO 440 J2=(J1+1),(ROWPOS(I+1)-1)
			IF ( COLPOS(J1) > COLPOS(J2)  ) THEN
				I0=COLPOS(J1)
				COLPOS(J1)=COLPOS(J2)
				COLPOS(J2)=I0
			END IF
 440		CONTINUE
 400	CONTINUE
	PRINT *, '   sparse structure sorted'

	END SUBROUTINE QR_STRU

C	*****************************************************************

	SUBROUTINE RECNZ(QPOS,K,LENQR,RR)
C	record a non-zero position in sparse structure, 
C	checking for max elements & unique entries (e.g. in case of symmetry)
      USE HOMPACK90_GLOBAL, ONLY: COLPOS, ROWPOS
	INTEGER, INTENT(IN):: K,LENQR,RR
	INTEGER, INTENT(IN OUT):: QPOS
	INTEGER I
!	LOGICAL OK  

	IF (QPOS>LENQR) STOP "more nonzeros in QR_STRU than MaxZeros allow"

C	check if K is already registered
	DO 10 I=ROWPOS(RR),QPOS-1
		IF (COLPOS(I)==K) RETURN
 10	CONTINUE

	COLPOS(QPOS)=K
	QPOS=QPOS+1	

	RETURN
	END SUBROUTINE RECNZ
