	SUBROUTINE RHO(A,LAMBDA,X,H)
C
C   EVALUATE  RHO(A,LAMBDA,X)  AND RETURN IN THE VECTOR  H .
C	MODEL: learning and forgetting, entry and exit
C
C	IMPLICIT NONE
      USE REAL_PRECISION, ONLY : R8
      REAL (KIND=R8), INTENT(IN):: A(:),X(:)
      REAL (KIND=R8), INTENT(IN OUT):: LAMBDA
      REAL (KIND=R8), INTENT(OUT):: H(:)
C
C
      INTEGER:: N

C     model parameters *********
	integer M, mmax
	real*8 vOut, sigma
	real*8 delta_beg, delta_end
	real*8 beta
	COMMON/ModelParams/beta,vOut,sigma,delta_beg,delta_end,M,mmax 
      real*8 delta, deltaVec(M)
C     cost parameters and vector
	real*8 rho_beg, rho_end, kappa
	COMMON/CostParams/rho_beg, rho_end, kappa
      real*8 cost(M),eta
     
C offsets in X and H vectors
	integer offP, offV
	integer offBell, offFOC
	COMMON/OFFSETS/offP,offV,offBell,offFOC

C     service variables
      integer i,j,k
      real*8 tempM(5)
C     conditional expectation, exponents and probabilities
      real*8 W(0:2), exps(0:2), D(0:2)
C other usefull stuff
      real*8 pij, demon

C     Functions
      real*8 TransProb
      real*8 GetV
C     CalcW is a routine

      N=SIZE(X)

C preliminary calculations
C current delta & depr. probability vector 
      delta = delta_beg + LAMBDA * (delta_end-delta_beg)
      do 10 i=1,M
         deltaVec(i)= 1.0 - (1.0-delta)**i
 10   continue
      
C     cost vector
      eta = dlog( rho_beg + LAMBDA * (rho_end-rho_beg) )/dlog(2.0D0)
      do 20 i=1,M
        cost(i)=kappa*(min0(i,mmax)**eta)
 20   continue

C exponent for alternative
	IF (vOut.gt.-101.0) THEN
	    exps(0)=dexp(vOut/sigma)
	ELSE
	    exps(0)=0.0D0
		print *, 'resetting outside good to -inf'
	ENDIF
C     
C	HOMOTOPY EQUATIONS - quick plug
C      do 110 i=1,M*M*2
C         H(i)=0
C 110     continue

C     state space loop (single continue) *******************
      do 31 i=1,M
      do 30 j=1,M
      
C       get prices, exponents and win probabilities
        pij = X(offP+ i+(j-1)*M)
        exps(1) = dexp(-pij/sigma)
        exps(2) = dexp(-X(offP+j+(i-1)*M)/sigma)
        demon = exps(0)+exps(1)+exps(2)
        
        do 40 k=0,2
           D(k)=exps(k)/demon
 40     continue
      
C     get conditional expectations
        call CalcW(i,j,W,X,M,deltaVec)
      
C     price equation (FOC*sigma/F1)
        H( offFOC + i+(j-1)*M ) = sigma - (1-D(1))*(pij - cost(i) 
     +                + beta*W(1)) + beta*( D(0)*W(0) + D(2)*W(2) )
C     value equation (Bellman)
        H( offBell + i+(j-1)*M ) = -getV(i,j,X,M) + D(1)*(pij-cost(i))
     +               + beta*( D(0)*W(0) + D(1)*W(1) + D(2)*W(2) )
      
 30   continue
 31   continue

C     Test stuff **********************
C         print *, 'Trans prob to (i-1,i,i+1,sum) for loose'
C      print *, 'Depr. probs(delta=',delta,'):'
C      print *,'cost vector'
C      print *, 'V(1:5,1:5)='

C      do 120 i=1,5
C         tempM(1)=TransProb(i-1,i,0,deltaVec,M)
C         tempM(2)=TransProb(i  ,i,0,deltaVec,M)
C         tempM(3)=TransProb(i+1,i,0,deltaVec,M)
C         tempM(4)=tempM(1)+tempM(2)+tempM(3)
C         print *, float(i), (tempM(j),j=1,4) 

C         print *, float(i), deltaVec(i)
C         print *, float(i), cost(i)

C         do 130 j=1,5
C            tempM(j)=GetV(i,j,X,M)         
C 130     continue
C         print *, (tempM(j),j=1,5),GetV(i,0,X,M) 

C 120   continue

C test of conditional expectation
c$$$      call CalcW(1,1,W,X,M,deltaVec)
c$$$      print *, 'W for (1,1): ', (W(j),j=0,2)
c$$$      call CalcW(4,5,W,X,M,deltaVec)
c$$$      print *, 'W for (4,5): ', (W(j),j=0,2)
c$$$      call CalcW(6,1,W,X,M,deltaVec)
c$$$      print *, 'W for (6,1): ', (W(j),j=0,2)
c$$$      call CalcW(2,20,W,X,M,deltaVec)
c$$$      print *, 'W for (2,20): ', (W(j),j=0,2)

      END

C service functions *****************************

C     Transtion probabilities function ****************************
      real*8 FUNCTION TransProb(i, old_i, outcome, deltaVec, M)
      integer i, old_i, outcome, M
      real*8 deltaVec(M), di

      di = deltaVec(old_i)

      if (outcome.eq.1) then
C      win
        if (old_i.lt.M) then
C        middle of state space
          if (i.eq.old_i+1) then
            TransProb = 1.0 - di
          elseif (i.eq.old_i) then
            TransProb = di
          else
            TransProb = 0.0
          endif
        
        else 
C         old_i==M
          if (i.eq.old_i) then
            TransProb = 1.0
          else
            TransProb = 0.0        
          endif
        
        endif 
    
      else 
C       loose
        if (old_i.gt.1) then 
C        interior of state space
          if (i.eq.old_i) then
            TransProb = 1.0 - di
          elseif (i.eq.old_i-1) then
            TransProb = di
          else
            TransProb = 0.0
          endif
        
        else
C        edge of state space
          if (i.eq.old_i) then
            TransProb = 1.0
          else
            TransProb = 0.0        
          endif
        
        endif

      endif
      
      END

C Get Value function from x-vector *******************************
      real*8 FUNCTION GetV(i,j,X,M)
      integer i,j,M
      DOUBLE PRECISION X(2*M*M)

C offsets in X and H vectors
	integer offP, offV
	integer offBell, offFOC
	COMMON/OFFSETS/offP,offV,offBell,offFOC

      if((i.ge.1).and.(i.le.M).and.(j.ge.1).and.(j.le.M)) then
         GetV = X( offV+ i + (j-1)*M )
      else
         GetV=0.0
      endif

      END

C Conditional expectations function *****************
C     W - receives the vector
      SUBROUTINE CalcW(i,j,W,X,M,deltaVec)
      integer i,j,M
      DOUBLE PRECISION X(2*M*M)
      real*8 deltaVec(M)
      real*8 W(0:2)

C     probablities if loosing
      real*8 p1(0:1), p2(0:1)
C     conditioning constants

C     functions
      real*8 TransProb
      real*8 GetV
C      real*8 TransProb(i, old_i, outcome, deltaVec, M)
C      real*8 GetV(i,j,X,M)

C      precalculate probabilites conditional on loosing
      p1(0) = TransProb(i-1,i,0,deltaVec,M)
      p1(1) = TransProb(i  ,i,0,deltaVec,M)
      p2(0) = TransProb(j-1,j,0,deltaVec,M)
      p2(1) = TransProb(j  ,j,0,deltaVec,M)

C       main calculation
      W(0) =  getV(i,  j  ,X,M)*p1(1)*p2(1)
     m      + getV(i-1,j  ,X,M)*p1(0)*p2(1)
     m      + getV(i  ,j-1,X,M)*p1(1)*p2(0)
     m      + getV(i-1,j-1,X,M)*p1(0)*p2(0)

      W(1) =  getV(i  ,j  ,X,M)*TransProb(i  ,i,1  ,deltaVec,M)*p2(1)
     m      + getV(i+1,j  ,X,M)*TransProb(i+1,i,1,deltaVec,M)*p2(1)
     m      + getV(i  ,j-1,X,M)*TransProb(i  ,i,1,deltaVec,M)*p2(0)
     m      + getV(i+1,j-1,X,M)*TransProb(i+1,i,1,deltaVec,M)*p2(0)

      W(2) =  getV(i  ,j  ,X,M)*p1(1)*TransProb(j  ,j,1,deltaVec,M)
     m      + getV(i-1,j  ,X,M)*p1(0)*TransProb(j  ,j,1,deltaVec,M)
     m      + getV(i  ,j+1,X,M)*p1(1)*TransProb(j+1,j,1,deltaVec,M)
     m      + getV(i-1,j+1,X,M)*p1(0)*TransProb(j+1,j,1,deltaVec,M)
      
      END
