/* Solve FOC = 0
FOC is hard-coded and is proven to be "single crossing" (>0 on the left)

Method is Bisection + Newton:
program checks if Newton update is between bounds,
if not, use bisection update.
In any case, update bounds.

compile in Matlab by "mex EpsZero.c", then call "EpsZero"
might have to tell Matlab where file is: "cd \matlab\work\foo" */

/*/ is the result of quick fixing of // (which unix compiler does not like )
to star-comments, like this -> */

/* include libraries*/
#include "mex.h" /*/ links to matlab*/
#include <math.h> /*/ math functions (exp)*/
#include "stdio.h"

double beta, v0, sigma;  /*/ globals, read directly from matlab*/
double p1, p2, c, wa, w, l; /*/ locals, passed as arguments*/
double e0, e2; /*/ components of FOC*/

/*/ subscripts of point matrix*/
const prc=0; /*/price "x"*/
const lev=1; /*/FOC level  "f(x)"*/
const der=2; /*/ derivative (SOC) "df/dx"*/

void evalFOC(double *point);
double FOC(double p1);

/*/ main function*/
double solveFOC(void)
{                     /*e-1234567890123456*/
  const double tolf = 0.000000000000001; /*/ error tolerance on FOC level*/
  /*/    const double tolf = 0.00000000000005; /* error tolerance on FOC level*/
  const maxIter = 200;    /*/ max # of iterations*/
    const srch_step = 2;    /* searching for 2nd init. point:
                            sigma*step^n */

    double left[3], right[3], current[3]; /*/ left & right points [p,lev, der]*/
    double srch_factor, z; /*/ misc vars*/
    char *IterType; /*/ "Newton-L, Newton-R, Bisect*/

    int i,j,count, newton;

    /*/ precalculate components of FOC*/
    e0 = exp( -v0/sigma );
    e2 = exp( -p2/sigma );



    /*/ find initial range*/
    current[prc] = p1;
    evalFOC(current);
    /*printf("Start: p=%5.4f \n",left[prc]);*/

    srch_factor = 0.5;
    if (current[lev]>0){ /*/ proper left,*/
      /*/ assign*/
        for(i=0;i<3;i++)
            left[i] = current[i];

        do {    /*/ search for right*/
            right[prc] = left[prc] + sigma*srch_factor;
            evalFOC(right);
            srch_factor *= srch_step;
        } while ( (right[lev]>0) && (srch_factor<10000) );

        if ( -right[lev] < left[lev] )
            for(i=0;i<3;i++)
                current[i] = right[i];
	/*/return right[prc]; /* test feature*/
    } else { /*/ if(left[lev]>0)-> it is actually right point*/
      /*/ flip points*/
        for(i=0;i<3;i++)
            right[i] = current[i];

        do { /*/ search for left*/
            left[prc] = right[prc] - sigma*srch_factor;
            evalFOC(left);
            srch_factor *= srch_step;
        } while ( (left[lev]<0) && (srch_factor<10000));

        if ( left[lev] < -right[lev]  )
            for(i=0;i<3;i++)
                current[i] = left[i];
	/*/return left[prc]; /* test feature*/
    } /*/if (left[lev]>0)*/

    /*if ( ( left[lev] <= 0 ) || (right[lev] >= 0))
        printf("!!! Bound search failed !!!\n");*/

    /*printf( " Initialize: LEFT: FOC(%5.4f) = %3.2e; RIGHT: FOC(%5.4f) = %3.2e \n",
            left[prc], left[lev], right[prc], right[lev] );*/

    count=1;

    while ( ( fabs(current[lev]) >= tolf ) && ( count++<=maxIter ) ) {
      /*/ Newton*/
      newton = 0;  /*/ indicates if newton update is good*/

      if( (left[lev] < -right[lev]) && (left[der] < 0) ) { /*/ or fabs([lev])*/
            current[prc] = left[prc] - left[lev]/left[der];
            if (  current[prc] < right[prc] ) /*/ (left < current ) by signle crossing*/
                newton = 1;

      } else if ( right[der] < 0 ) { /*/ includes right==left, as right is better*/
            current[prc] = right[prc] - right[lev]/right[der];
            if ( left[prc] < current[prc] ) /* (current < right) by single crossing*/
                newton = 2;
        }

      /* Bisection update*/
        if ( !newton ) {
            current[prc] = left[prc] + (right[prc] - left[prc])
                            * left[lev] / (left[lev] - right[lev]);
        }

        /*/ evaluate function & update bounds*/
        evalFOC(current);
        if ( current[lev]>0 ) /*/ update left*/
            for(i=0;i<3;i++)
                left[i] = current[i];
        else
            for(i=0;i<3;i++)
                right[i] = current[i];

        /*switch(newton){
            case 0: IterType="Bisect-n"; break;
            case 1: IterType="Newton-L"; break;
            case 2: IterType="Newton-R"; break;
            default: IterType="!!! wrong value"; break;
        }

        printf( "%2d.%s: LEFT: FOC(%5.4f) = %3.2e; RIGHT: FOC(%5.4f) = %3.2e, error=%3.2e \n",
                count, IterType, left[prc], left[lev], right[prc], right[lev], fabs(current[lev]) ); */
    } /*/ while ((fabs(current[lev])>=tolf ) && ( count<=maxIter ) )*/

    /*if (fabs(current[lev])>=tolf)
        printf("!!! MAX interations reached !!!\n");
    else
        printf("!!! Success !!!"); */

    return current[prc];
}

/*/ FOC calculation*/
void evalFOC(double *point) /*/ [p,lev, der]*/
{
    double F, F0, e1;

    /*/ pre-calculate probabilites (no need for separate f-n)
    //already have: e0 = exp(-v0/s); e2 = exp(-p2/s);*/
    e1 = exp( point[prc]/sigma );
    F0 = 1 / ( 1 + e0/e1 + e2*e0 ); /*/exp((-p1-v0)/s)+exp((-p2-v0)/s)*/
    F  = 1 / ( 1 + e1/e0 + e1*e2 ); /*/exp(( p1+v0)/s)+exp(( p1-p2)/s));*/

    /*/ level of FOC*/
    point[lev] = sigma - (1-F) * ( point[prc] - c + beta * w )
        + beta * ( F0 * wa + (1 - F - F0) * l) ;

    /*/ derivative of FOC = SOC*/
    point[der] = F * point[lev] / sigma - 1;
    /*printf("   evalFOC: F=%5.4f, F0=%5.4f, p=%5.4f, FOC=%5.4f, dFOC=%5.4f \n",
           F, F0, point[prc], point[lev], point[der]);   */
}

double FOC(double p1) /*/ original FOC - just for testing*/
{
    double F, F0, z;

    /*/ pre-calculate probabilites (no need for separate f-n)*/
    F0 =  1 /( 1 + exp( (-p1 - v0)/ sigma )
                 + exp( (-p2 - v0)/ sigma ) );

    F  = 1 / (      exp( (p1 + v0)/ sigma )
              + 1 + exp( (p1 - p2)/ sigma ) );

    z = sigma - (1-F) * ( p1 - c + beta * w )
        + beta * ( F0 * wa + (1 - F - F0) * l) ;

    /*printf("   raw FOC: F=%5.4f, F0=%5.4f, p=%5.4f, FOC=%5.4f \n",
           F,F0, p1, z ); */

    return z;
}


/*/ "Gateway function" - converts from Matlab to C vars
// also "entry point" - runs when .dll is called*/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs,
                 const mxArray *prhs[])
{
	double *z;

    if (nrhs != 6)
        mexErrMsgTxt("FOC_ac: Six inputs required.");

    /*/ read inputs*/
    p1 = *mxGetPr(prhs[0]); /*/ 1st argument*/
    p2 = *mxGetPr(prhs[1]);
    c  = *mxGetPr(prhs[2]);
    wa = *mxGetPr(prhs[3]);
    w  = *mxGetPr(prhs[4]);
    l  = *mxGetPr(prhs[5]);

    /*/ init output*/
    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);

    /*/ read paramenter*/
	beta = *mxGetPr(mexGetVariablePtr("global", "beta"));
    v0 = *mxGetPr(mexGetVariablePtr("global", "vOut"));
    sigma = *mxGetPr(mexGetVariablePtr("global", "sigma"));

    z = mxGetPr(plhs[0]);
    z[0] = solveFOC(); /*/ run main function*/

}
