/* WLEc.c - conditional expectations, linux version
 Learning curve model
 duopoly, alternative good, Exit&Entry
 calculates Wa, W, L, from V directly in Matlab
 MATLAB: type "mex WLEc.c" to compile
*/


/* ***** Note the 0-based indexing in C arrays:
i,j = 0:M ***** */

/* include libraries */
#include "mex.h" /* links to matlab */
#include <math.h> /* math functions (exp) */

/* declare global variables */
double delta1, delta2, lambda; /* depreciation params */
int M, win, loose; /* # of states, outcome constants */
double *V, *W, *L, *Wa; /* Value f-n and cond. expectations */

/* declare functions */
double TransLCc(int i1, int i0, int outcome);
double getV(int i, int j); /* access extended V */

/* main function */
void WLEc(void)
{
    int i,j;
    /* probs of {i-1,i}, cond. on loosing */
    double p1[2],p2[2];

    for (i=0;i<M;i++){
    for (j=0;j<M+1;j++){
      /* precalculate probabilites */
        p1[0] = TransLCc(i-1,i,loose);
        p1[1] = TransLCc(i  ,i,loose);
        p2[0] = TransLCc(j-1,j,loose);
        p2[1] = TransLCc(j  ,j,loose);

        /* main calculation */
        Wa[i+j*(M+1)] = getV(  i,  j)*p1[1]*p2[1]
                      + getV(i-1,  j)*p1[0]*p2[1]
                      + getV(  i,j-1)*p1[1]*p2[0]
                      + getV(i-1,j-1)*p1[0]*p2[0];

        W[i+j*(M+1)] = getV(  i,  j)*TransLCc(i,i,win)*p2[1]
                     + getV(i+1,  j)*TransLCc(i+1,i,win)*p2[1]
                     + getV(  i,j-1)*TransLCc(i,i,win)*p2[0]
                     + getV(i+1,j-1)*TransLCc(i+1,i,win)*p2[0];

        if (j<M)  /* competitor operates */
         L[i+j*(M+1)] = getV(  i,  j)*p1[1]*TransLCc(j,j,win)
                      + getV(i-1,  j)*p1[0]*TransLCc(j,j,win)
                      + getV(  i,j+1)*p1[1]*TransLCc(j+1,j,win)
                      + getV(i-1,j+1)*p1[0]*TransLCc(j+1,j,win);
        else  /* competitor exited => can't win */
          L[i+j*(M+1)] = 0;


    }
    }
}

double getV(int i, int j)
{ /* note the 0-indexing in C */
    if ( i>=0 && i<=M && j>=0 && j<=M )
        return V[i+j*(M+1)];
    else
        return 0;

}

/* the Transitional probabilities function
  replicated in TransE.m - called by TransMatE_duo.m */
double TransLCc(int i1, int i0, int outcome)
{
    double di;

    /* Process inactive state */
    if ( i0 == M){ 
        if ( i1 == i0 )
            return 1;
        else
            return 0;
    } 

    /* pre-calc probability of forgetting */
    di = lambda*delta1 + (1-lambda)*( 1 - pow(1-delta2,i0+1) );

    if ( outcome == win){
        if ( i0 < (M-1) ){ /* interior of staying */
            if (i1 == i0+1)
                return 1-di;
            else if ( i1 == i0 )
                return di;
            else
                return 0;
        } else { /* i.e. i0 == M-1 -- boundary staying */
            if ( i1 == i0 )
                return 1;
            else
                return 0;
        } 

    } else {/* if ( outcome == win) - i.e. loose */
        if ( i0 > 0 ){
            if (i1 == i0)
                return 1-di;
            else if ( i1 == i0-1 )
                return di;
            else
                return 0;
        } else {/* if ( i0 > 1 ) - i.e. i0==1 */
            if ( i1 == i0 )
                return 1;
            else
                return 0;
        } /* if ( i0 > 1 ) else */
    } /* if ( outcome == win) else */

    return 0; /* just in case, this point is never reached */

}


/* "Gateway function" - converts from Matlab to C vars
   also "entry point" - runs when .dll is called */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs,
                 const mxArray *prhs[])
{
    /* read parameters */
    delta1 = *mxGetPr(mexGetVariablePtr("global", "delta1"));
    delta2 = *mxGetPr(mexGetVariablePtr("global", "delta2"));
    lambda = *mxGetPr(mexGetVariablePtr("global", "lambda"));
            /* ... & convert to integer */
    M = *mxGetPr(mexGetVariablePtr("global", "M"));
    win = *mxGetPr(mexGetVariablePtr("global", "win"));
    loose = *mxGetPr(mexGetVariablePtr("global", "loose"));

    /* read/init matrices V, W, L, Wa */
    V = NULL; Wa=NULL; W=NULL; L=NULL;
    V = mxGetPr(mexGetVariablePtr("global", "V")); /* value */
    Wa = mxGetPr(mexGetVariablePtr("global", "Wa")); /* CE(alt wins) */
    W = mxGetPr(mexGetVariablePtr("global", "W")); /* CE(1 wins) */
    L = mxGetPr(mexGetVariablePtr("global", "L")); /* CE(2 wins) */

    /* Run the calculation */
    WLEc(); /* run main function */

}

/* The Catch is to make sure C subscripts
 do not go out-of-range of Matlab variable
 -  mxGetNumberOfElements(*mxArray);
 -  mxGetNumberOfDimensions(*mxArray); */
