% duopolyEE3ne.m - No Exit or Entry: Calc V, p
% version 3: G-S, regular state space ordering, dampen after calc, separate V

% global variables - see master file for comments
% "own" globals of this script
global iter % iteration counter, for plot_duoLC
global tolV tolp tols% tolerance levels

waitAK = 0; % 1 if want to wait-for-button-press after each iteration plot update

if (X>=-a) | (S>=-b)
    error('DuopolyEE3ne: parameterization allows for entry/exit ( X>=-a or S>=-b)')
end 

% init iterations
iter = 1; done = 0;
tolV = zeros(1,maxiter); tolp = zeros(1,maxiter); tols = zeros(1,maxiter);
 %V(M+1,:) = 0; % inactive - special treatment
V1=V; U = zeros(M+1);
stay=ones(M+1);
p(1:M,M+1)=0; p(M+1,:) = +inf; 
warning off MATLAB:fzero:UndeterminedSyntax

% pre-calculate conditional expectations (Wa, W, L) -  in iteration, it is done after updating V
run WLEc % general .c/.dll procdeure
% run WL % matlab code for no exit-only

tic;
while ~done
    
    p0=p; p1=p;
    %%% Compute optimal policy  p1
    for p_i=1:stepsP
        
        for i=1:M
        for j=1:M
            %%% Numeric solution to FOC: choose one of three
                %%% our own C-coded solution
            p1(i,j) = solveFOCc( p1(i,j), p1(j,i), cost(i), Wa(i,j), W(i,j), L(i,j) );
%                 %%% matlab's FZero on C-coded FOCc.c
% 	        p1(i,j) = fzero( @FOCc, p1(i,j), [],  p1(j,i), cost(i), Wa(i,j), W(i,j), L(i,j) ); 
%                 %%% Matlab's FZero on Matlab script FOC.m 
% 	        p1(i,j) = fzero( @FOC, p1(i,j), [],  p1(j,i), cost(i), Wa(i,j), W(i,j), L(i,j) ); 
	
            if BanPred % satisfy predation constraint
                if j<M+1 % competitor exists
                    cj=cost(j);
                else % competitor is out
                    cj=inf;
                end
                p1(i,j) = max( p1(i,j), min( cost(i), cj ) );
            elseif NonNegP
                p1(i,j) = max( p1(i,j), 0 );
            end% if BanPred
            
            
        end % l
        end % k
        
        % apply dampening
        p = p + dampP*(p1-p);
        p(M+1,:) = +inf;
    end % p_i
    

    %%% Update value f-n V1
    V0=V; % W0 = W; % save prev-iteration matrix
        
    for v_i =1:stepsV
       % continuation value 
        for i=1:M 
        for j=1:M 
            F = Fsale( p(i,j), p(j,i) );  % precalculate probabilities
            F0 = Fsale0( p(i,j), p(j,i) );
                % main calculation
            V(i,j) = F .* ( p(i,j) - cost(i) ) + beta .* ( F0 .* Wa(i,j) ...
                        + F .* W(i,j) + (1 - F - F0) .* L(i,j) );
        end % j
        end % i
	
		% calculate conditional expectations (Wa, W, L)
		run WLEc % general .c/.dll procdeure
% 		run WL % matlab code for no exit-only
    end % v_i
    
    %%% calculate tolV, tolp, -> done
    tolV(iter) = max(max(abs( V(1:M,1:M) - V0(1:M,1:M)))) ./(1+max(max(abs( V0(1:M,1:M) ))));
    tolp(iter) = max(max(abs( p1(1:M,1:M) - p0(1:M,1:M) )))./(1+max(max(abs( p0(1:M,1:M) ))));
    
    % check convergence
    if (tolV(iter) < tol) & (tolp(iter) < tol) 
      done = 1;
      result = 'Desired precision reached. THE END';
      Convd = '';
    end

    % check maximum iterations
    if iter >= maxiter 
        done = 1;
        result = 'Max number of iterations reached. THE END';
        Convd = sprintf('  NO CONVERGENCE: tol(p)=%.1e tol(V)=%.1e', ...
            iter, tolp(iter), tolV(iter) );
    end
    
    % display oteration summaryin command window
    if (~mod(iter-1,rateD) | done) & rateD ~= 0  
        disp(sprintf('iter. %3d: tol(V)=%.2e  tol(p)=%.2e',  ...
            iter, tolV(iter), tolp(iter) )) 
    end

    % update iteration plot (if "it is time")
    if (~mod(iter,rateP) | done) & rateP ~= 0  
        run plotEE_Iter
        if waitAK
           k = waitforbuttonpress; % k=0 mouse, k=1 keyboard
           if k == 0
               waitAK = 0;
           end
        end
    end
    
    %%% move to next iteration
    iter = iter+1;
     
    if iter==100 & dampP2>0
        dampP = dampP2
    end

end % while done

disp(sprintf('%s (time elapsed - %f)', result, toc))

% Calculate functions
dp = p(1:M,1:M)-(p(1:M,1:M))'; % price differential ( us - them )
marga = p(1:M,1:M+1) - repmat( cost([1:M]'), 1, M+1 ); % absolut margin
   maxdP = max(max(abs(dp)));

Profit = zeros(M+1); Profits = zeros(M+1); 
ProbC = FSale(p,p');
Profit(1:M,1:M+1) = marga(1:M,1:M+1) .* ProbC(1:M,1:M+1);
Profits = Profit + Profit';

%clear irrelevent vars from list
clear F F0 V0 i j k s v p1 p0;
