% homNE90_Start.m -- runs the homotopy:
% 	  -  loads saved equilibrium that will serve a starting point of homotopy,
% 	  -  writes the equilibrium and parameter values into binary file (hom_start.dat),
% 	  -  calls the executable compiled from Fortran90 file,
% 	  -  generates the "name" of the run (fileMod variable),
% 	  -  initiates output processing by running homNE90_read.m and hom_steps.m.
      
clear all; % variables 
global M
    M=30; %%%%%%%%%%%%%%%%%%
run InitParamsEE % define & init globals

%%%%%% Parameter control (re-define for ease of use)
drate=0.05; beta=1/(1+drate); % set drate=inf for beta=0
sigma = 1.0; % differention rate

%%% Equilibirum selection 
FileMod = 'gf'; Eqbm = 'Flat';
% FileMod = 'gt'; Eqbm = 'Trenchy';
% FileMod = 'g3'; Eqbm = '(Xtra)trenchy';

%%% range of parameters for homotopy
delta2=0.0;
endDelta=0.01;

rho=0.85;
endRho = rho;

    
    %%% Build Run Header (for display and plots)
    BP='NBP'; % no price restrictions in homotopy
    sMonCap = ''; sMonCap2 = '';

    RunHead = sprintf(['E&E, X=%2.1f a=%2.1f S=%2.1f b=%2.1f; disc.=%g; d(p)=%g d(c)=%3g mix=%2.1f(0=prop);' ...
                       'c0=%3.1f sigm=%3.2f rho=%3.2f %s %s'], ...
        X, a, S, b, drate, delta2, delta1, lambda, -vOut, sigma, rho, BP, sMonCap2);
    PlotHead2=sprintf('NE: %s Eqbm, \\rho=%g, \\delta=%g, ',... 
                   Eqbm, rho, delta2 );
   
    %%%% Build File header (filename)
    if vOut > -inf % alternative price
        alt = sprintf('v%d',round(-vOut*10));
    else
        alt = 'vInf';
    end
    
    if drate < inf % discount parameter beta
        dsc = sprintf('b%d',round(drate*100));
    else
        dsc = 'bInf';
    end
    
    if M==20
        MM='';
    else
        MM=sprintf('%d',M);
    end

    if mmax==round(M/2)
        smmax='';
    else
        smmax=sprintf('m%d',mmax);
    end

    if round(delta2*100)*10==round(delta2*1000)
        sDelta2=sprintf('%d',round(delta2*100));
    else
        sDelta2=sprintf('%03d',round(delta2*1000));
    end
    
    FileHead = sprintf('DuoEE%s%s_x%da%ds%db%dl%ddp%sdc%d%ss%dr%d%s%s%s',...
            MM, smmax, round(X*10),round(a*10), round(S*10), round(b*10), ...
            round(lambda*10), sDelta2,round(delta1*100),alt, ...
            round(sigma*10),round(rho*100),BP,dsc, sMonCap);

%%% load file %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    filename = ['GridDat/' FileMod FileHead '.mat'];
    if exist(filename)==2
       load(filename,'V', 'p');
    else
       disp(['Starting file not found:' FileMod FileHead '.mat'])
return
    end

    
    
%%% save parameters & X-vector into a file
if exist('hom_start.dat','file')
    delete('hom_start.dat'); % delete old file, since some systems cannot overwrite it
end
fid = fopen('hom_start.dat','w'); % open file for write
if fid==-1
    error('File open failed')
end
c1=fwrite(fid, [M mmax beta kappa vOut sigma ... 
        delta2 endDelta rho endRho], 'float64');
        
X= [ reshape(p(1:M,1:M), M*M,1); ...
     reshape(V(1:M,1:M), M*M,1) ];
c2=fwrite(fid,X,'float64');
c3=fclose(fid);

% check for success 
if (c1==10) & ( c2==length(X) )
    disp('params & X-vector written to file')
else
    error('params & X-vector writing failed')
end

if c3~=0
    error('file close failed')
end

%%% uncomment this line if you only want to generate starting value file
% return % stops execution of this script

%%% call homotopy %%%
%% Windows (make sure path and filename are correct)
eval('!LBD.exe') % 
% eval('!LBD.exe > batch.out') % output into a batch file
%% Unix (a.out is default filename of compiled fortran code)
% eval('!./a.out | batch.out') % "./" is necessary

%%% generate run name for use below - 
% build up fileMod string from pieces (string = row vector of integers)
if vOut==-10
  fileMod='';
else
  fileMod=alt;
end
fileMod=[smmax fileMod];
if sigma~=1
   fileMod = [fileMod sprintf('s%02d',round(sigma*10))];
end
if endRho==rho
  fileMod=[fileMod sprintf('r%02d',round(rho*100))];
else
  fileMod=[fileMod sprintf('r%02dt%02d',round(rho*100),round(endRho*100))];
end
if endDelta==delta2
  fileMod=[fileMod 'd' sDelta2 dMod];
else
  fileMod=[fileMod 'd' sDelta2 sprintf('t%02d',round(endDelta*100))];
end
disp(['File mod = ' fileMod])

%%% zip up homotopy output & clean up
    % eval('!foo') executes system command 'foo'
%% Unix version,  -jq = (j)unk path info, (q)uiet operation (no feedback on screen)
% eval(['!cp hom_start.dat hom_start_' fileMod '.dat'])
% eval('!rm hom_start.dat') % must remove it or next run will not work
% eval(['!zip -jq h9_' fileMod ' HomXpt/hom_0[0-4]*.dat'])
% eval(['!zip -jq h9_' fileMod ' HomXpt/hom_*.dat'])
% eval(['!mv batch.out ' fileMod '.out'])
%% Windows version
eval(['!copy hom_start.dat hom_start_' fileMod '.dat'])
eval(['!zip -jq h9_' fileMod ' HomXpt\hom_*.dat'])
eval(['!move batch.out ' fileMod '.out'])


%%% convert step files into matlab ones - homNE90_read.m
% return % you must comment out first four lines in homNE90_read.m before you comment this out
% fileMod=... - already created
deltaRng=[delta2 endDelta]; % starting and ending values of delta
rhoRng=[rho endRho]; % rho that comes with the run (if homotopy was on rho, should modify this file
XEnds=0; % 1=exlude start/end of run when computing number of equilibria 
 % (should be on for one of two runs that make up a loop
run homNE90_read % run the conversion routine

%%% compute herf  - call to hom_steps.m
run hom_steps