% batch processing of homotopy step files (generated by hom_read.m)
%%% Automomous run setup - comment out if running from hom_start.m
% 	clear all;
% 	close all;
% 
% 	% some stuff to use herf, Lim/Trans distr code from iterative part
% 	global M mmax
% 	M=30; mmax=15;
% 	run InitParamsEE % define & init globals
% 	
% 	%%% run choice
% 	fileMod='v1000r85d0t01'; % run code, same as used in hom_read.m
%%% End of Automomous run setup - comment out if running from hom_start.m

%%% steps on which the action is takes
load(['HomRes/DeltaRho_' fileMod],'steps') 
steps=[1 10:10:steps steps];
% steps=[13 56]; % just couple of states to plot

%%% actions to take
doPlot = 0; % plot
doHerfs = 1; % Compute herfs (Limiting,Maximum, T=8, T=32)
doConv = 0; % run iterative algorithm as a check stability
saveRef =0; % save last step into hom_ref.mat

%%% plot control
global PlotHead PlotHead2 PlotDate PlotPage% titles of plot
PlotDate='3-Jul-07';
psFile =''; % enter filename for postscript printing
PlotPage=1; % number on 1st page (0 to skip numbering)
PlotHead=''; % just need that

% initialize
hParam=zeros(1,3);
if exist(psFile)==2
    delete(psFile) % output PostScript file
end
deltas=[]; % store of deltas for each step
sHerfs=zeros(0,4);
stay=ones(M+1);
disp(' ');

for si=1:length(steps)
    hom_step=steps(si);
    haveFile=0;
   
    filename=['HomDat/' fileMod '/' sprintf('step_%05d.mat', hom_step)];
    if exist(filename,'file')
        load(filename, 'p','V','delta2','rho'); %,  ...
        disp(sprintf('Step %2d, delta=%.6f', ...
                     hom_step, delta2 ));
		eta = log(rho)/log(2);
        deltas(si,1)=delta2;        
        % pad p&V so it can be used with general EE code
		p(:,M+1)=0; p(M+1,:)=inf; 
		V(:,M+1)=0; V(M+1,:)=0;
        stay(1:M+1,1:M+1)=1.0;
    else
        
        disp(sprintf( 'step %d: file not found', hom_step ));
continue        
    end
    
	if saveRef% save into hom_ref.mat
        pref=p;
        Vref=V;
        save('hom_ref','pref','Vref');
    end
    
    if doConv
		p(:,M+1)=0; p(M+1,:)=inf; pst=p;
		V(:,M+1)=0; V(M+1,:)=0; Vst=V;
	    rateD=10; rateP = 20; % display and plot rate
	    tol = 1e-17; maxiter = 5000; % tolerance and max iterations
	    dampP = 1.0; dampP2 = 0; dampS = 0.2; dampS2=0;% dampening, Policy and Stay, S2 kicks in after iter#100
	    stepsV=1; stepsP = 1; % iterate on value function
	    innTol = 1e-15; innMax = 50; % inner iteration tolerance and max iters
        
        run duopolyNE % regular NE procedure
        
        disp('Absolute differences in p,V - with reference value: ')
        load('hom_ref'); % must contain 'pref' 'Vref'
        disp(max(max(abs( p(1:M,1:M) - pref(1:M,1:M) ))));
        disp(max(max(abs( V(1:M,1:M) - Vref(1:M,1:M) ))));
        disp('     - with starting value')
        disp(max(max(abs( p(1:M,1:M) - pst(1:M,1:M) ))));
        disp(max(max(abs( V(1:M,1:M) - Vst(1:M,1:M) ))));

        PlotHead2 = [PlotHead2 ' After iterations ' Convd];
        
        if maxiter>100
            figure; h=axes;
            plot([1:maxiter],[tolp(1:maxiter);tolV(1:maxiter)])
            set(h, 'YScale','log');
            legend('Price','Value',0);
            title('Relative iterative tolerances')
        end
        
    end
        
    if doHerfs | doPlot % (need dynamics for for plotting
        P=[]; Pe=[]; DL=[]; DT=[]; lHerfP=[]; dStats=[];
        load(filename, 'DL','DT','T', ...
              'lHerfP', 'lHerfP0', 'maxHerfP', 'amaxHerfP', 'dStats'); %,  ...
        
        if isempty(DL) | isempty(DT)
			p(:,M+1)=0; p(M+1,:)=inf;
			V(:,M+1)=0; V(M+1,:)=0;
            dp = p(1:M,1:M)-(p(1:M,1:M))'; % price differential ( us - them )
            marga = p(1:M,1:M+1) - repmat( cost([1:M]'), 1, M+1 ); % absolut margin
               maxdP = max(max(abs(dp)));
            Profit = zeros(M+1); Profits = zeros(M+1); 
            ProbC = FSale(p,p');
            
			run markEE_Lim; limT=0;
			EE_symmetry(0); 
			T=[8,32]; 
            run markEE_Trans;
            T=[T inf];
			save(filename, ... 
                'DL', 'limT', 'lHerfP', 'lHerfP0', 'lHerfV', 'maxHerfP', 'amaxHerfP',...
               'lRevers', 'lLeadP', 'lFlwP', 'lExpP', 'lMode', ...
               'DT', 'T', '-append');
         end
         if isempty(dStats) % statistic for transients
			Distr(:,:,1)=DT(:,:,1);
			Distr(:,:,2)=DT(:,:,2);
% 			Distr(:,:,3)=DL; 
			dStats = DistrStatsEE(Distr); clear Distr;
			for ki=1:length(T)
	            dStats(ki).T = T(ki);
			end
			save(filename, 'dStats', '-append');

		end
        
        sHerfs=[sHerfs; lHerfP maxHerfP dStats(1).HerfP dStats(2).HerfP];
    end
    
    if doPlot

        PlotHead2=sprintf('Run %s, Step %2d, delta=%.4f', ...
                      fileMod, hom_step, delta2 );
    
        plotNE_ResTrans
    end
    
end

if doHerfs
    save(['HomRes/Herf_' fileMod],'steps','deltas','sHerfs'); 
    
    dOff = 0.01;
    deltaTick=[0 0.01 0.03 0.05 0.1 0.2 1.0];
    HOff = -0.49;
    HerfTick=[0.5 0.51 0.55 0.7 1.0]'; % tickmark labels
    
    dHerf = log(sHerfs(:,1) + HOff);
    dMaxH = log(sHerfs(:,1) + HOff);
    dDelta = log(deltas + dOff);
    HerfTick2 = HerfTick;
    HerfTick = log(HerfTick + HOff);
    deltaTick2 = deltaTick;
    deltaTick = log(deltaTick + dOff);

    figure;
    ha=axes;
    hl = plot( dDelta, [dHerf dMaxH] ); % Lim and Max
    set(hl(2),'LineStyle',':');
		v=axis; 
        v(3:4)=log([0.5 1.0] + HOff); axis(v);
        set(ha, 'YTick',HerfTick);
        set(ha, 'YTickLabel',HerfTick2);
        v(1:2)=log([0.0 1.0] + dOff); axis(v);
        set(ha, 'XTick',deltaTick);
        set(ha, 'XTickLabel',deltaTick2);
    xlabel('\delta'); 
    ylabel('H^\infty, H^\wedge'); %'Rotation',90,
    legend('H^\infty', 'H^\wedge')
        
end

% if length(steps>1) & doPlot % close multiple graphs by single click
%     waitforbuttonpress;
%     close all;
% end

