% Starting point - duopoly, No entry or exit, alternative
% Learning curve model. Besanko Doraszelski Satterthwaite (2004) 
% 2 firms, ENTRY & EXIT disabled; written for Matlab 6.5

clear all; % variables - globals and not
close all; % figures (graphs)

%%%%%%%%% Set/Calculate Parameters
% if notation is diferent from paper, original is given in brackets

global M % size of state space
    M=30; %%%%%%%%%%%%%%%%%%
run InitParamsEE % define & init globals
   mSurplus=[]; mWelfare=[]; ForceSurf=[];
global RunHead PlotHead FileHead % title of run, plots (+ data/page), 
global PlotHead2 PlotDate PlotPage
    LoopHead=''; PlotHead2 =''; PlotDate='';

%%%%%% Parameter control (re-define for ease of use)
%%% Key parameters of the run(s)
% rho_val = [0.95 0.85 0.75 0.65 0.55 0.45 0.35 0.25 0.15 0.05]; % full grid
rho_val = [0.85]; % use for subset of grid, or custom values

% dlt_val = [0 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.1 0.3 0.5 0.7 0.9]; % full grid
dlt_val = [0.0 0.01]; % use for subset of grid, or custom values
% can select multiple values to use one as starting value for another


% misc model parameters - no need to change
drate=0.05; beta=1/(1+drate); % discounting: set drate=inf for beta=0
sigma = 1; % differentiation

% inputs to transient distribution computation
i0 = 1; j0 = 1; state0 = '(1,1)'; % intial state, numbers and label
T2=[8 32]; % transient distributions periods


%%%% Equilibirum selection - uncomment the proper line
FileMod = 'gf'; Eqbm = 'Flat';
% FileMod = 'gt'; Eqbm = 'Trenchy';
% FileMod = 'g3'; Eqbm = '(Xtra)trenchy';

%%%% ACTION control (1 = YES, 0 = NO)%%%%%%%%
Calculate = 1; % 0 to load from file, 1 for iterative, 
%       if file exists, calculation starts from it
%       if file does not exits, calc start from scratch, or eqbm left over from prev. run
Dynamics = 0; % Derive lim.distr., Herfs, etc.: 0=skip, 
%           1=derive, but skip if Calculation has instant or no convergence , 
%           2=derive in anyncase
doSave = 1; % save everything into a file: 0=skip,  2=save in any case
%           1=derive, but skip if Calculation has instant or no convergence,
doPlot = 0; % plot the results
SpecialOp = 0; % collect indicators (herfs, etc.), compare and copy
%%%  Compare and copy options: comment out to cancell
%     FileComp='gt'; % compare to another equilibirum file
%     FileDest = ['../' FileMod]; % copy file, replaces FileMod

%%% Computation control
rateD=25; rateP = 10; % display and plot frequency, steps, =0 for none
tol = 1e-15; maxiter = 2000; % relative tolerance and max iterations {e.g. 1e-5, 500}
dampP = 0.4; dampP2 = 0; %% dampening, Policy if dampP2>0, it kicks in after iter#100 {1.0, 0}
stepsV=1; stepsP = 1; % repeat of inner iterations (value and policy updates); {1}

%%% PAGE control - page # and date on plots
PlotPage = 1; %%%% set to >1 if continuing a sequence of plots
PlotDate='3-Jul-07'; 

%%% main part starts

% technical inits
runs=[]; crun =1;
runs = [runs length(dlt_val)]; %current run and number of runs
runs = [runs length(rho_val)]; % current run and number of runs

% init the loops
LogTol = zeros(runs);
if SpecialOp
    %  intialize indicator arrays
    if exist('FileComp')==1
        CompP = repmat(NaN,runs); CompV = repmat(NaN,runs);
    end
    gMaxH = zeros(runs); % max Herf 
    gLimH = zeros(runs); % lim Herf 
    gConv = zeros(runs);
    
end % if SpecialOp

%%% paramter loops

ActFlags = [Calculate Dynamics doSave];

for r_i = 1:length(rho_val)
    rho = rho_val(r_i); eta = log(rho)/log(2); 

for d_i = 1:length(dlt_val)
    delta2 = dlt_val(d_i); %length(dlt_val)+1- 
    
%%% extra loops for experiments
% for drate=[0.05 0.02] %[0.05 0.1 0.2 0.3 0.4 0.5 0.7 0.9]
%     beta=1/(1+drate);
% for vOut=[-10 -100]
%     vOut = v0_val(v_i); 
% for sigma=[0.1 0.045]
% for mmax=[15 30];
    %%% define actions 
    Calculate = ActFlags(1); Dynamics = ActFlags(2); doSave = ActFlags(3);
    
    %%% Build Run Header for filename, display and plots
    if BanPred 
        BP='BP';
    elseif NonNegP
        BP='NNP';
    else
        BP='NBP';
    end
    
    if isempty(MonCap)
        sMonCap = ''; sMonCap2 = '';
    else
        sMonCap = sprintf('Cap%d', round(MonCap*10));
        sMonCap2 = sprintf('Mon.Cap=%3.2f', MonCap);
    end

%     RunHead = sprintf(['E&E, X=%2.1f a=%2.1f S=%2.1f b=%2.1f; disc.=%g; d(p)=%g d(c)=%3g mix=%2.1f(0=prop);' ...
%                        'c0=%3.1f sigm=%3.2f rho=%3.2f %s %s'], ...
%         X, a, S, b, drate, delta2, delta1, lambda, -vOut, sigma, rho, BP, sMonCap2);
    RunHead = sprintf(['NE, rho=%3.2f delta=%g; beta=%.3f ' ...
                       'v0-c0=%3.1f sigm=%3.2f %s %s'], ...
        rho, delta2, beta, vOut+v_gd, sigma, BP, sMonCap2);
    PlotHead2=sprintf('%s Eqbm, \\rho=%g, \\delta=%g',... 
                   Eqbm, rho, delta2);
   
    %%%% Build File filename
    if vOut > -inf % alternative price
        alt = sprintf('v%d',round(-vOut*10));
    else
        alt = 'vInf';
    end
    
    if drate < inf % alternative price
        dsc = sprintf('b%d',round(drate*100));
    else
        dsc = 'bInf';
    end
    
    if M==20
        MM='';
    else
        MM=sprintf('%d',M);
    end

    if round(delta2*100)*10==round(delta2*1000)
        sDelta2=sprintf('%d',round(delta2*100));
    else
        sDelta2=sprintf('%03d',round(delta2*1000));
    end
    
    % template "DuoEE{M}_x##e##l##dp##dc##v###s##r##BPb##.mat"
    FileHead = sprintf('DuoEE%s_x%da%ds%db%dl%ddp%sdc%d%ss%dr%d%s%s',...
            MM, round(X*10),round(a*10), round(S*10), round(b*10), ...
            round(lambda*10), sDelta2,round(delta1*100),alt, ...
            round(sigma*10),round(rho*100),BP,dsc);

    disp('******************************************'); 
    disp(sprintf('run %d of %d: %s', ...
                crun, prod(runs), PlotHead2))
    disp(RunHead)

    %%% calculate/load the outputs
    P=[]; Pe=[]; DL=[];% clear transition matrices to force re-calculation 
    HaveFile=0; DT=[]; dStats=[];
    
    if Calculate
        if (exist(['GridDat/' FileMod FileHead '.mat'])==2 ) 
            disp('   Starting from existing file')
            load(['GridDat/' FileMod FileHead],'V','p','stay')
        else
            disp('starting from equilibirum in memory ')
        end
        
        %%%% CALCULATE
        run duopolyNE % No exit or entry only
        LogTol(d_i,r_i) = max([ tolV(iter-1) tolp(iter-1) tols(iter-1) ]);
        
        if iter<=2 % instant convergence, no changes necessary
            Dynamics=max(0,Dynamics-1); doSave=max(0,doSave-1);
        end
        if length(Convd)>2 % no convergence, cancell recalc 
            Dynamics=max(0,Dynamics-1);  
        end
        HaveFile=1;
    else % load
        if exist(['GridDat/' FileMod FileHead '.mat'])==2
            load(['GridDat/' FileMod FileHead],'delta1', 'delta2', 'lambda', 'M', 'mmax', 'beta', 'vOut', ...
                   'sigma', 'tol', 'iter', 'kappa', 'rho', 'eta', 'BanPred', ... % 'RunHead', 
                   'Convd', 'V', 'p', 'stay', 'X', 'a', 'S', 'b',  ...  %'dampP', 'dampS',
                   'DL', 'limT', 'maxdP', 'ForceSurf',  ... %'i0', 'j0', 'state0', 'VV', 'U', 
                   'lHerfP', 'lHerfP0', 'lHerfV', 'maxHerfP', 'amaxHerfP',...
                   'lWelfare', 'lSurplus', 'lWelfareD', 'lSurplusD', 'lDyears', ... % 'XC','SS',
                   'lRevers', 'lLeadP', 'lFlwP', 'lExpP', 'lMode', ...
                   'DT', 'T', 'dStats');
            
            % calculate non-saved vars (necessary - do not comment out!!!)
            dp = p(1:M,1:M)-(p(1:M,1:M))'; % price differential ( us - them )
            marga = p(1:M,1:M+1) - repmat( cost([1:M]'), 1, M+1 ); % absolut margin
               maxdP = max(max(abs(dp)));
            Profit = zeros(M+1); Profits = zeros(M+1); 
            ProbC = FSale(p,p');
            Profit(1:M,1:M+1) = marga(1:M,1:M+1) .* ProbC(1:M,1:M+1);
            Profits = Profit + Profit';
            
            HaveFile=1;
            if length(Convd)>2
                disp(Convd)
            else
                disp('File loaded. Convergence OK')
            end
        else
            disp('File not found')
        end
    end % if calculate
    
    if Dynamics & HaveFile
      % limiting distribution
      run markEE_Lim; limT=0;
      % transient distributions
      T=[8 32]; % periods to compute 
      run markEE_Trans
      % symmetry measures (Herfs) and Welfare indicators
      EE_symmetry(0); % (1) to display detailed plot of measures
      EE_welfExp(0);
    end
    
    if doSave & HaveFile 
        save( ['GridDat/' FileMod FileHead], 'delta1', 'delta2', 'lambda', 'M', 'mmax', 'beta', 'vOut', ...
               'sigma', 'tol', 'iter', 'kappa', 'rho', 'eta', 'RunHead', 'BanPred', ...
               'Convd', 'V', 'p', 'stay', 'X', 'a', 'S', 'b', 'dampP', 'dampS', ... 
               'DL', 'limT', 'i0', 'j0', 'state0', 'maxdP', 'ForceSurf', ...
               'lHerfP', 'lHerfP0', 'lHerfV', 'maxHerfP', 'amaxHerfP',...
               'lWelfare', 'lSurplus', 'lWelfareD', 'lSurplusD', 'lDyears', 'XC','SS',...
               'lRevers', 'lLeadP', 'lFlwP', 'lExpP', 'lMode', ...
               'DT', 'T');
           %    'IDfail', 'IIDfail', 'IIIDtext', 'IDfp', 'IIDfp', 'VV', 'U', ...
        disp(['  file saved (filename modifier:' FileMod ')']);

    end % if doSave
    
    crun=crun+1;

    if SpecialOp & HaveFile 

        %%% tabulate indicators
		if length(Convd)==0 
            gConv(d_i,r_i)=1;
		end
		
        % record summary statistics
		if ~isempty(lHerfP) & HaveFile % old limherf
            gLimH(d_i,r_i)=lHerfP;
            gMaxH(d_i,r_i) = maxHerfP; 
		end
         
		%%% make copy
        if exist('FileDest')==1 & HaveFile
            copyfile(['GridDat/' FileMod FileHead '.mat'], ... 
                     ['GridDat/' FileDest FileHead '.mat' ])
            disp(['  file copied to modifier: ' FileDest ]);
        end
        
		%%% compare
		if exist('FileComp')==1 & HaveFile
            if ~exist('CompAll','var')
                CompAll = zeros(0,4);
            end
			if exist(['GridDat/' FileComp FileHead '.mat'])==2
				Vc=V; pc=p; 
                load(['GridDat/' FileComp FileHead],'V','p');
				CompP(d_i,r_i)= max(max(abs( pc(1:M,1:M) - p(1:M,1:M) ))) / ( 1+ max(max(abs( p(1:M,1:M) ))) );
				CompV(d_i,r_i)= max(max(abs( Vc(1:M,1:M) - V(1:M,1:M) ))) / ( 1+ max(max(abs( V(1:M,1:M) ))) );
                CompAll=[CompAll; rho vOut CompP(d_i,r_i) CompV(d_i,r_i)];
				V=Vc; p=pc; 
            else
				CompP(d_i,r_i)= NaN;
				CompV(d_i,r_i)= NaN;
            end
		end

    end % if SpecialOp
    
    if doPlot & HaveFile
        plotNE_ResTrans
    end

end % for param
end % for param-2
% end % for param-3


clear depr alt delta BP crun runs LoopHead eol k F F0 v;
clear Calculate PlotDetail PlotWIS PlotComb PlotCustom;

ccc=cost(1:M); % for data tables

