function blocks = partition(P)
% PARTITION : Partition the state space of a Markov chain into its closed communicating 
%				  classes C(1),...,C(k) and the union of all the other communicating classes T.
%
% blocks = partition(P);
%
% Input:  P         is the n by n transition matrix for an n-state Markov chain.
%
% Output: blocks    is an n-vector of integers 0:k that labels the states of the 
%						  Markov chain such that blocks(i)=l, where l=1,...,k, if state i 
%						  belongs to C(l) and blocks(i)=0 if state i belongs to T.

% Determine communicating classes.

blocks = components(P>0);

% Determine closed communicating classes.
n = size(P,1);
k = max(blocks);
for l=1:k
   states = find(blocks==l);
   Q = P(states,setdiff(1:n,states));
   if sum(Q(:))>0
      blocks(states) = 0;
   end
end

% Relabel closed communicating classes.
labels = unique(blocks(find(blocks>0)));
j = 1;
for l=labels
   blocks(find(blocks==l)) = j;
   j = j+1;
end
